function [] = PlotSirQC(inputfolder,ProjDesc,dFileStr,Campaign)
%% Plot long echogram:
freport = fopen([inputfolder,filesep,ProjDesc,'QCReport.txt'],'w');
fprintf(freport,'SIR QC Report\n Generated: %s\n\n',datestr(now,'mm/dd/yyyy HH:MM:SS'));
fprintf(freport,'\t\tFile Name\t\t\t\t\t\t File Size (MBytes)\n');
fprintf('\t\tFile Name\t\t\t\t\t\t File Size (MBytes)\n');
for jj = 1:length(dFileStr)
    fprintf(freport,'%s\t\t\t%0.1f\n',dFileStr(jj).name,dFileStr(jj).bytes/1024/1024);
    fprintf('%s\t\t\t%0.1f\n',dFileStr(jj).name,dFileStr(jj).bytes/1024/1024);
end
fprintf(freport,'\n\n');
fprintf('\n\n');
filekey='*SIR*.mat';
% clims=[5,90];                    % dB
if strcmpi(Campaign,'AN01')
    ylims=[0, 650];                    % m
elseif  (strcmpi(Campaign,'AN02') || strcmpi(Campaign,'AN03'))
    ylims=[0, 875];                    % m
end
cmap='gray';                       % valid colormap string
invertcaxis=1;                     % 0 or 1
c_air=3e8;                         % m/s
c_ice=1.68e8;                      % m/s
cutterH=1;                         % horizontal decimation
pagesize=[15,8];                   % inches
resolution=600;                    % dpi

% Find input files:
inputfiles=dir([inputfolder,filekey]);
numfiles=length(inputfiles);
if numfiles==0
    error('Could not locate input files.')
end

% Loop through and plot input files:

for ii=1:numfiles
    load([inputfolder,inputfiles(ii).name],'sirfftsave','Range','echoTimeutcOut','numfftAvg','numCohAvg','NoisePwr','SurfPwr','SurfInds')
    report.surfPwr(ii) = SurfPwr;
    report.noisePwr(ii) = NoisePwr;
    report.utc(ii) = echoTimeutcOut(1);
    report.surfRange(ii) = mean(Range(SurfInds));
    report.fileName{ii} = inputfiles(ii).name;
    
    % Determine data within elevation range:
    InRange= abs(Range)>ylims(1)& abs(Range)<ylims(2);
    
    %radecho(InRange,1:cutterH:end) = filter(ones(1,numAvg)/numAvg,1,abs(radecho(InRange,1:cutterH:end)),[],2);
    % plot data:
    figure(ii);
    if invertcaxis==1
        imagesc(unix2matlabtime(echoTimeutcOut),Range(InRange),-20*log10(abs(sirfftsave(InRange,1:cutterH:end))));
    else
        imagesc(unix2matlabtime(echoTimeutcOut),Range(InRange),20*log10(abs(sirfftsave(InRange,1:cutterH:end))));
    end
    titletext={['SIR QC: ', ProjDesc(1:9),datestr(unix2matlabtime(echoTimeutcOut(1)),' mmddyy HH:MM')];['numCohAvg = ',num2str(numCohAvg),', numfftAvg = ',num2str(numfftAvg)]};
    eval(['colormap(',cmap,')'])
    xlabel('Flight Time, UTC')
    xlim([min(unix2matlabtime(echoTimeutcOut)),max(unix2matlabtime(echoTimeutcOut))]);
    datetick2('x','HH:MM:SS','keeplimits','keepticks')
    set(gca,'ydir','reverse')
    ylabel('Range in Ice, m')
    title(titletext)
    set(gca,'TickDir','out')
    ylim(ylims);
    % Draw figure:
    drawnow
    
    [~,figname,~] = fileparts(inputfiles(ii).name);
    figname = [inputfolder,figname,'_QC.pdf'];
    set(gcf,'PaperSize',pagesize)
    set(gcf,'PaperPosition',[0,0,pagesize])
    print('-dpdf',figname,['-r',num2str(resolution)])
end
fprintf('Printing Report...\n');
for bb = 1:numfiles
        fprintf(freport,'UTC Time %s : File %s \n',datestr(unix2matlabtime(report.utc(bb)),'mm/dd/yyyy HH:MM:SS'),[report.fileName{bb}]);
        fprintf(freport,'Surface Power  : %0.1f dB\t\t Range\t Air: %0.1f m (%0.1f ft) \t Ice Equivalent (c = 1.68e8): %0.1fm\n',...
            report.surfPwr(bb),report.surfRange(bb)*3/1.68,report.surfRange(bb)*3/1.68*3.28,report.surfRange(bb));
        fprintf(freport,'Noise Power    : %0.1f dB\n\n',report.noisePwr(bb));
        fprintf('UTC Time %s : File %s \n',datestr(unix2matlabtime(report.utc(bb)),'mm/dd/yyyy HH:MM:SS'),[report.fileName{bb}]);
        fprintf('Surface Power  : %0.1f dB\t\t Range\t Air: %0.1f m (%0.1f ft) \t Ice Equivalent (c = 1.68e8): %0.1fm\n',...
            report.surfPwr(bb),report.surfRange(bb)*3/1.68,report.surfRange(bb)*3/1.68*3.28,report.surfRange(bb));
        fprintf('Noise Power    : %0.1f dB\n\n',report.noisePwr(bb));
end
fclose(freport);
disp('Done')