% Tejendra Dhakal
% 09/14/2015
% LDEO, Columbia University

% This Script parses the SIR file and plots
% Dependencies:
% Fb2Range.m - Function that computes range from frequncy,chirpperiod etc
% unix2matlabtime.m
% fixSirFile.m
% datetick2.m
% PlotSirQC.m
%%
clc; clear all; close all;
tic
%% Processing Flags
gpu_process = 0;
if(gpu_process)
    reset(gpuDevice(1))
end

Campaign = 'AN02'; % 'AN01', or 'AN02' or 'AN03'

SaveData = 1;

numCohAvg = 32;                         % numbers of chirps to average before fft
numfftAvg = 2;                         % numbers of chirps to average after fft
numAvg = numCohAvg*numfftAvg;

motionComp = 0;
if strcmp(Campaign,'AN01')
    surfbreakrange = [4900:5600];         
    NoiseInds = [20:100];
elseif (strcmpi(Campaign,'AN02') || strcmpi(Campaign,'AN03'))
    surfbreakrange = [3560:4500];      
    NoiseInds = [20:100];
else
    error('Invalid Campaign')
end

winSizeSurfFilt = 50;
overwriteoutput = 0;
%% File Parameters
numTry = 2;
while(numTry)
    inputfolder = uigetdir([],'Select the Folder with SIR QC Data');                  % folder containing input data (include trailing '/' or '\')
    numTry = numTry -1;
    if(inputfolder == 0)
        if(numTry > 0)
            ed = errordlg({'You Are Wasting My Time!';'Select the QC Data Folder or Else..'},'Path Error');
            uiwait(ed,60);
        else
            ed = errordlg({'Have it Your Way..';'Good Bye!'},'Path Error');
            uiwait(ed,5);
        end
    else
        break;
    end
end
inputfolder = [inputfolder,filesep];
outputfolder=[inputfolder,'Processed_QC',filesep];                           % folder to send output data (include trailing '/' or '\')

% inputfolder= uigetdir([],'Select Folder with SIR QC Data');                  % folder containing input data (include trailing '/' or '\')
% if(inputfolder == 0)
%     errordlg({'Do you want to QC SIR or NOt?';'Run the Prog and select the Folder.'})
%     exit;
% end
% inputfolder = [inputfolder,'\'];
% outputfolder=[inputfolder,'Processed_QC\'];
fileFieldSep = '_';
dateField = 3;           % Indices of datestamp within input file names (two integers)
timeField = 4;          % Indices of timestamp within input file names (two integers)
campaignField = 1;
flightNoField = 2;
file_period = 10;                    % file length in seconds

rawFileDateFormat = 'yyyymmdd';
rawFileTimeFormat = 'HHMMSS';

%% Radar Parameters

if strcmpi(Campaign,'AN01')

    fs = 12.5e6;        % Sampling Frequency
    tau = 750e-6;        % Chirp Period
    bw = 600008082;     % Chirp bandwidth Hz
    framelen = 9352;    % number of samples collected for each chirp
    dataWidth = 2;      % Each sample is 4 Bytes
    sigRange = 3:9350; % discard first 15 samples, sample format: Marker(2^32) Marker (2^32)
    metadatalen = 0; % starttime(unixsec) ppscount(unixsec) clkcount(sampleclk) echocount
elseif (strcmpi(Campaign,'AN02') || strcmpi(Campaign,'AN03'))
    fs = 50e6;        % Sampling Frequency
    tau = 250e-6;        % Chirp Period
    bw = 599996441;     % Chirp bandwidth Hz
    framelen = 12474;    % number of samples collected for each chirp
    dataWidth = 4;      % Each sample is 4 Bytes
    sigRange = 250:12400; % discard first 15 samples, sample format: Marker(2^32) Marker (2^32)
    metadatalen = 4; % starttime(unixsec) ppscount(unixsec) clkcount(sampleclk) echocount
end
tsamp = linspace(0,tau-1/fs,fs*tau);
fft_period = 1*tau;   % chirp period is

% starttime(unixsec) ppscount(unixsec) clkcount(sampleclk) echocount
tsamp = tsamp(sigRange);
fftdatalen = numel(sigRange);
conversion_factor = 2.415/2^16; % adc conversion into volts
%% Constants
c_ice = 1.68e8;     % speed of light in ice m/s
c = 299792458;      % speed of light in vacuum m/s
%%
dFileStr = dir([inputfolder,'*.sir']);
numDataFiles = length(dFileStr);
if numDataFiles==0
    error('Data Files Not Found!')
end
% Sort data files by timestamp:
% for nfiles = 1:numDataFiles
%     dFileStr(nfiles).utc=86400*...
%         (datenum([dFileStr(nfiles).name(filenamedateinds(1):filenamedateinds(2)),...
%         dFileStr(nfiles).name(filenametimeinds(1):filenametimeinds(2))],'yyyymmddHHMMSS')...
%         -datenum('01-01-1970 00:00:00','dd-mm-yyyy HH:MM:SS'));
% end
for nfiles = 1:numDataFiles
    fileNameFields = strsplit(dFileStr(nfiles).name,fileFieldSep);
    dFileStr(nfiles).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[rawFileDateFormat,rawFileTimeFormat]));
end


[sv,sx] = sort([dFileStr.utc]);
dFileStr = dFileStr(sx);
fTime = [dFileStr.utc];

dTime = diff(fTime);
batchind = find(dTime >11);
numbatches = numel(batchind)+1;
batchmin(1) = 1;
if isempty(batchind)
    batchmax(1) = numel(fTime);
else
batchmax(1) = batchind(1);
batchmin(numbatches) = batchind(end)+1;
batchmax(numbatches) = numDataFiles;

for bb=2:numbatches-1
    batchmin(bb) = batchind(bb-1)+1;
    batchmax(bb) = batchind(bb);
end
end
ProjDesc = [fileNameFields{campaignField},'-',fileNameFields{flightNoField},'-',fileNameFields{dateField}];
FlightNo = fileNameFields{flightNoField};
% Make output folder, if necessary:
% Make output folder, if necessary:
if exist(outputfolder,'dir')==0
    mkdir(outputfolder)
end
%% Compute Range and Frequency bins
NFFTr = 2^nextpow2(fftdatalen);
fbins = fs/2*linspace(0,1,NFFTr/2+1)';
fbins_full = fs/2*linspace(-1,1,fftdatalen)';
twowaytime = fbins*tau/bw;
InRange = 1:NFFTr/2;
fbins = fbins(InRange);
Range = Fb2Range(bw,tau,fbins);
maxLinesperFile = ceil(max([dFileStr.bytes]/dataWidth/framelen))+ numAvg;
filesperbatch = max(batchmax-batchmin)+1;
maxLinesperBatch = filesperbatch*maxLinesperFile;
% Pre-allocate
if(gpu_process == 1)
    tempFFTdata = gpuArray(zeros(fftdatalen,ceil(maxLinesperFile/numCohAvg)));
end
if strcmp(Campaign,'AN01')
    tempFileData = uint16(zeros(maxLinesperFile*framelen,1));
    excessData = uint16(zeros(framelen,numAvg));
elseif (strcmpi(Campaign,'AN02') || strcmpi(Campaign,'AN03'))
    tempFileData = uint32(zeros(maxLinesperFile*framelen,1));
    excessData = uint32(zeros(framelen,numAvg));
end
tempData = zeros(maxLinesperFile*fftdatalen,1);
excessframes = 0;
% This creates logical index for entire file to extract data in one go
% dataind = repmat(logical([zeros(2,1);zeros(4,1);zeros(sigRange(1)-7,1);ones(fftdatalen,1);zeros(framelen-sigRange(end),1)]),maxLinesperFile,1);
dataind = repmat(logical([zeros(2,1);zeros(4,1);zeros(sigRange(1)-7,1);ones(fftdatalen,1);zeros(framelen-sigRange(end),1)]),maxLinesperFile,1);
% metadataind = repmat(logical([zeros(2,1);ones(metadatalen,1);zeros(sigRange(1)-metadatalen-3,1);zeros(fftdatalen,1);zeros(framelen-sigRange(end),1)]),maxLinesperFile,1);
% metadata= zeros(metadatalen,maxLinesperFile);
sig = zeros(fftdatalen,maxLinesperFile);
sif2 = zeros(NFFTr,maxLinesperFile);
sirfftout = zeros(length(InRange),ceil(maxLinesperBatch/numAvg)+numAvg);
sirPwrEcho = zeros(1,ceil(maxLinesperFile/numAvg)+numAvg);
if(motionComp == 1)
    H_MC = zeros(fftdatalen,ceil(maxLinesperFile));
    del_R = zeros(fftdatalen,ceil(maxLinesperFile));
    del_tau = zeros(fftdatalen,ceil(maxLinesperFile));
end

fftwin = hann(fftdatalen);
fftwinMat = repmat(fftwin,1,maxLinesperFile);
tsampMat_full = repmat(tsamp,1,ceil(maxLinesperFile));

isfirstfile = 1;
lastutc = dFileStr(1).utc;

% Initialize batch start and end files:
minfNum=batchmin;
maxfNum=batchmax;

for batch=1:numbatches
    fprintf('Processing batch %d of %d\n',batch,numbatches)
    isfirstfile = 1;
    for fnum = minfNum(batch):maxfNum(batch)
        if(dFileStr(fnum).bytes/dataWidth <framelen)
            warning('Skipping Truncated File %s\n Size smaller than fft_period: %d',...
                dFileStr(fnum).name,dFileStr(fnum).bytes/dataWidth);
            continue
        end
        % check if last file is older than 10 seconds.. if it is get rid of
        % the excess data
        if((fnum-minfNum(batch)) > 1 && excessframes >0)
            if((dFileStr(fnum).utc - lastutc) <= 11)
                tempFileData(1:excessframes*framelen) = excessData(1:excessframes*framelen);
            end
        else
            excessframes = 0;
            lastutc = dFileStr(minfNum(batch)).utc;
        end
        
        numframesInFile = dFileStr(fnum).bytes/dataWidth/framelen;
        if(mod(numframesInFile,1)>0)
            numframesInFile = floor(numframesInFile);
            warning('Partial Frames Found in File %s - Skipping',dFileStr(fnum).name);
            fprintf('Recovering Frames...\n');
            if strcmp(Campaign,'AN01')
                dFileStr(fnum).bytes = fixSirFile([inputfolder,dFileStr(fnum).name],framelen);
            elseif (strcmpi(Campaign,'AN02') || strcmpi(Campaign,'AN03'))
                 dFileStr(fnum).bytes = fixSirFile_stacked([inputfolder,dFileStr(fnum).name],framelen);
            end
            continue;
        end
        lastutc = dFileStr(fnum).utc;
        framesToRead = (numframesInFile) - mod(excessframes + numframesInFile,numAvg);
        fid1 = fopen([inputfolder,dFileStr(fnum).name],'r');
        fprintf('Processing file %d of %d\n',fnum-minfNum(batch)+1, maxfNum(batch)-minfNum(batch)+1)
        
        numframes = excessframes + framesToRead;
        if strcmpi(Campaign,'AN01')
        tempFileData(excessframes*framelen+1:numframes*framelen) = ...
            fread(fid1,(framesToRead*framelen),'uint16','b');
        excessframes = numframesInFile - framesToRead;
        excessData(1:excessframes*framelen) = ...
            (fread(fid1,excessframes*framelen,'uint16','b'));
        elseif (strcmpi(Campaign,'AN02') || strcmpi(Campaign,'AN03'))
                    tempFileData(excessframes*framelen+1:numframes*framelen) = ...
            fread(fid1,(framesToRead*framelen),'uint32','b');
        excessframes = numframesInFile - framesToRead;
        excessData(1:excessframes*framelen) = ...
            (fread(fid1,excessframes*framelen,'uint32','b'));
        end
        fclose(fid1);
        % Remove mean and convert to voltage
        tempData(1:numframes*fftdatalen) = detrend(double(tempFileData(dataind(1:numframes*framelen))),'constant')*conversion_factor;
        % compute time for each echo
        echoTimeutc = linspace(dFileStr(fnum).utc,dFileStr(fnum).utc+file_period,numframes/numAvg);
        echoTimeutc_coh = linspace(dFileStr(fnum).utc,dFileStr(fnum).utc+file_period,numframes/numCohAvg);
        echoTimeutc_full = linspace(dFileStr(fnum).utc,dFileStr(fnum).utc+file_period,numframes);
        if(isfirstfile == 1)
            lastnumFrames = 0;
            echoTimeutcOut = zeros(0);
            sirPwrOut = zeros(0);
            isfirstfile = 0;
            mintime = 86400*floor(echoTimeutc_coh(1)/86400);
            firsttime=(echoTimeutc_coh(1)-mintime)/86400;
            if (motionComp == 1)
                init_Alt = interp1(GPSUTC,Alt,echoTimeutc_coh(1));
            end
        end
        
        % calculate H for motion compensation
        if(motionComp == 1)
            del_R(:,1:numframes) = repmat((interp1(GPSUTC,Alt,echoTimeutc_full) - init_Alt),fftdatalen,1);
            del_tau(:,1:numframes) = 2*del_R(:,1:numframes)/c_ice;
            H_MC(:,1:numframes) = exp(-1j*(f0.*del_tau(:,1:numframes) + ...
                2*pi*Kr.*tsampMat_full(:,1:numframes).*del_tau(:,1:numframes) - ...
                pi*Kr*(2*tau.*del_tau(:,1:numframes) - del_tau(:,1:numframes).^2)));
        end
        %% hilbert transform
        sig(:,1:numframes/numCohAvg) = hilbert(sig(:,1:numframes/numCohAvg));
        %% Motion compensation
        if(motionComp == 1)
            sig(:,1:numframes/numCohAvg) = sig(:,1:numframes/numCohAvg).* H_MC(:,1:numframes/numCohAvg);
        end
        %% Coherent Averaging
        if(numCohAvg>1)
            sig(:,1:numframes/numCohAvg) = mean(reshape(tempData(1:numframes*fftdatalen),fftdatalen,numCohAvg,numframes/numCohAvg),2);
        else
            sig(:,1:numframes/numCohAvg) = reshape(tempData(1:numframes*fftdatalen),fftdatalen,numframes/numCohAvg);
        end
        
        %% Power Calc
        sirPwrEcho(1:numframes/numCohAvg) =  rms(sig(:,1:numframes/numCohAvg));
        %% FFT
        if(gpu_process == 1)
            tempFFTdata(:,1:numframes/numCohAvg) = fftwinMat(:,1:numframes/numCohAvg).*sig(:,1:numframes/numCohAvg);
            sif2(:,1:numframes/numCohAvg) = gather(fft(tempFFTdata(:,1:numframes/numCohAvg),NFFTr));
        else
            sif2(:,1:numframes/numCohAvg) = fft(fftwinMat(:,1:numframes/numCohAvg).*sig(:,1:numframes/numCohAvg),NFFTr);
        end
        if(numfftAvg>1)
            sirfftout(:,lastnumFrames + [1:numframes/numAvg]) = reshape(mean(reshape(sif2(InRange,1:numframes/numCohAvg),numel(InRange),numfftAvg,numframes/numCohAvg/numfftAvg),2),length(InRange),numframes/numCohAvg/numfftAvg);
        else
            sirfftout(:,lastnumFrames + [1:numframes/numAvg]) = sif2(InRange,1:numframes/numCohAvg);
        end
        %% cat matrices for saving
        sirPwrOut = [sirPwrOut sirPwrEcho(1:numframes/numAvg)];
        echoTimeutcOut = [echoTimeutcOut echoTimeutc];
        lastnumFrames = lastnumFrames+ numframes/numAvg;
    end
    sirfftsave = sirfftout(:,1:lastnumFrames);
    [~,SurfInds]=max(filter(ones(1,winSizeSurfFilt)/winSizeSurfFilt,1,abs(sirfftsave(surfbreakrange,:)),[],2));
    SurfInds = surfbreakrange(1)+SurfInds-1;
    SurfPwr = 20*log10(mean(mean(abs(sirfftsave(SurfInds-5:SurfInds+5,:)))));
    NoisePwr = 20*log10(mean(mean(abs(sirfftsave(NoiseInds,:)))));
    % Create output batch name:
    outputname = [FlightNo,'_','SIR','_',datestr(unix2matlabtime(echoTimeutcOut(1)),[rawFileDateFormat,'_',rawFileTimeFormat,'_']),num2str(batch),'.mat'];
    % Save output file:
    if exist([outputfolder,outputname],'file')==0 || overwriteoutput==1
        save([outputfolder,outputname],'dFileStr','sirfftsave','ProjDesc','fs','fbins','NFFTr','tsamp','Range','echoTimeutcOut','numfftAvg','numCohAvg','twowaytime','NoisePwr','SurfPwr','SurfInds');
    else
        warning(['Warning: Output File Already Exists, Batch Not Saved, Output File: ',outputname])
    end
end
% Final human communication:
disp('Done Processing!')
disp('Generating Plots...')
PlotSirQC(outputfolder,ProjDesc,dFileStr,Campaign);
toc
