function [] = sirDecimateLevel1(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the Processed data, decimates and concatenates the
% files for bed picking etc.
% Processsing.
%   - Decimates the pulseCompressed data by a given factor
%   - Concatenates files to given period
%   - options for averaging and smoothing available before decimation
% Initialize the settings and variables

try
    fprintf('************** sirDecimateLevel1.m ****************\n')
    
    if nargin <1
        configScript = 'sirProcessConfig.m';
        logFile = 'sirProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('sirDecimateLevel1:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    elseif nargin <2
        configScript = newConfig;
        logFile = 'sirProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('sirDecimateLevel1:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    else
        configScript = newConfig;
        localDiary = false;
    end
    
    run(configScript);
    tic
    echoLen = numel(Range);
    maxnumTraces = ceil(filePeriod/prfPreSar*1.1*filesperbatch) + bedPickTracesPerFile;
    tmpecho = zeros(echoLen,maxnumTraces);
    tmpLat = zeros(1,maxnumTraces);
    tmpLon =  zeros(1,maxnumTraces);
    tmpSurfInds =  zeros(numSurfPicks,maxnumTraces);
    tmpPlaneHeight =  zeros(1,maxnumTraces);
    tmpFlightElev =  zeros(1,maxnumTraces);
    
    tmputc = zeros(1,maxnumTraces);
    
    if strcmp(bpSubsampleOption,'average')
        sigOpt = 1;
    elseif strcmp(bpSubsampleOption,'decimate')
        sigOpt = 2;
    elseif strcmp(bpSubsampleOption,'smooth')
        sigOpt = 3;
        bedpickFilt = ones(1,bedPickDecimate)/bedPickDecimate;
    else
        error('sirDecimateLevel1:InvalidSubSampleOptions','Invalid "bpSubsampleOption" \n Check your setting')
    end
    %% Work
    outfileCount = 0;
    for ll = 1:numel(LineNum)   %% process each line separately
        bedPickDecimatedTime = now;
        configFileCopied = false;
        
         %% Generate input/output paths
        fftDecimatedLineDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,LineNum{ll},filesep,decimatedDir,filesep,dataDir,filesep];
        fftProcessedLineDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,LineNum{ll},filesep,dataDir,filesep];
        processLogPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,LineNum{ll},filesep,decimatedDir,filesep,logDir,filesep];
        if exist(fftProcessedLineDataPath,'dir')==0
            warning('sirDecimateLevel1:noLevel1Dir','Level1 Line %s Directory does not exists!',LineNum{ll});
            continue;
        end
        
        %% Check input directory and sort data files.
        level1Files=dir([fftProcessedLineDataPath,'*',LineNum{ll},'*',Sensor,'*.mat']);
        numfiles=numel(level1Files);
        if(numfiles == 0)
            warning('sirDecimateLevel1:NoSirLevel1Files','No level1 %s files found!\nMoving on!',LineNum{ll})
            continue;
        else
            for ii = 1:numfiles
                fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
                try
                    old_fileformat = false;
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    old_fileformat = true;
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                    warning('sirDecimateLevel1:oldFileNameFormat','File names are in old format, run dicePulseCompress to rename these files')
                    warning('off','sirDecimateLevel1:oldFileNameFormat');
                end
            end
            [~,sx] = sort([level1Files.utc]);
            level1Files = level1Files(sx);
        end
        
        %% Check for existing output directories and data (if any)
        if exist(fftDecimatedLineDataPath,'dir')==0
            mkdir(fftDecimatedLineDataPath)
        elseif overwriteoutput == 1
            warning('sirDecimateLevel1:DecimatedDirExists','Line %s Decimated directory already exists!\n All existing files will be deleted!',LineNum{ll});
            delete([fftDecimatedLineDataPath,'*',LineNum{ll},'*',Sensor,'*.mat']);
            delete([fftDecimatedLineDataPath,'sir*onfig*.m']);
        else
            warning('sirDecimateLevel1:DecimatedDirExists','Line %s Decimated directory already exists - it will be skipped!\n Set "overwriteoutput = 1" to force processing',LineNum{ll});
            continue;
        end
        
        [~,configFilename,~] = fileparts(configScript);
        copyfile([configFilename,'.mat'],[fftDecimatedLineDataPath,configFilename,'.mat'])
        fprintf('sirDecimateLevel1: Processing Line %s\n',LineNum{ll})
        
    %% Compute batches to take care of gaps in data
        fTime = [level1Files.utc];
        dTime = diff(fTime);
        batchind = find(dTime >filesperbatch*filePeriod*1.1);
        numbatches = numel(batchind)+1;
        batchmin(1) = 1;
        if isempty(batchind)
            batchmax(1) = numfiles;
        else
            batchmax(1) = batchind(1);
            batchmin(numbatches) = batchind(end)+1;
            batchmax(numbatches) = numfiles;
        end
        for bb=2:numbatches-1
            batchmin(bb) = batchind(bb-1)+1;
            batchmax(bb) = batchind(bb);
        end
        if numbatches>1
             warning('sirDecimateLevel1:FilesNotContiguous','Files are not contiguous!');
        end
        minfNum=batchmin;
        maxfNum=batchmax;
        
        %% Start decimating
        load([fftProcessedLineDataPath,level1Files(1).name],'Campaign','twtt','InRange','FlightNo')
        Time = twtt(InRange);
        for batch=1:numbatches
            numRemainingTraces = 0;
            for ii = minfNum(batch):maxfNum(batch)
                load([fftProcessedLineDataPath,level1Files(ii).name],'radecho','echotimeutc','lat','lon','hell','layerInd','prodLevel')
                fprintf('sirDecimateLevel1: Processing File %d of %d\n',ii,numfiles);
                numTraces = size(radecho,2);
                if(numRemainingTraces > 0)
                    tmpecho(1:echoLen,1:numRemainingTraces+numTraces) = [tmpecho(1:echoLen,1:numRemainingTraces),radecho];
                    tmputc(1:numRemainingTraces+numTraces) = [tmputc(1:numRemainingTraces),echotimeutc];
                    tmpLat(1:numRemainingTraces+numTraces) = [tmpLat(1:numRemainingTraces),lat];
                    tmpLon(1:numRemainingTraces+numTraces) = [tmpLon(1:numRemainingTraces),lon];
                    tmpSurfInds(:,1:numRemainingTraces+numTraces) = [tmpSurfInds(:,1:numRemainingTraces),layerInd];
                    tmpFlightElev(1:numRemainingTraces+numTraces) = [tmpFlightElev(1:numRemainingTraces),hell];
                    
                    numRemainingTraces = numRemainingTraces + numTraces;
                else
                    tmpecho(1:echoLen,1:numTraces) = radecho;
                    tmputc(1:numTraces) = echotimeutc;
                    tmpLat(1:numTraces) = lat;
                    tmpLon(1:numTraces) = lon;
                    tmpSurfInds(:,1:numTraces) = layerInd;
                    tmpFlightElev(1:numTraces) = hell;
                    
                    numRemainingTraces = numTraces;
                end
                
                                      % backward compatibility
                    if ~iscell(prodLevel.desc)
                        oldProdLevel = prodLevel;
                        clear prodLevel;
                        prodLevel.name = oldProLevel.name;
                        prodLevel.desc = {oldProdLevel.desc};
                    end
                    if ~ismember('time',fieldnames(prodLevel))
                    for pp = 1:numel(prodLevel.desc)
                        prodLevel.time{pp} = '?';
                    end
                    end
                    prodLevel.name = 'level1a';
                    prodLevel.desc{numel(prodLevel.desc)+1} = 'bedpickDecimated';
                    prodLevel.time{numel(prodLevel.time)+1} = bedPickDecimatedTime;
                    
                while(numRemainingTraces >= bedPickTracesPerFile)
                    if(sigOpt ==1)
                        Data = reshape(mean(reshape(tmpecho(1:echoLen,1:bedPickTracesPerFile),...
                            echoLen,bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),2),...
                            echoLen,bedPickTracesPerFile/bedPickDecimate);
                        UTC_time = mean(reshape(tmputc(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        Latitude = mean(reshape(tmpLat(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        Longitude = mean(reshape(tmpLon(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        Elevation = mean(reshape(tmpFlightElev(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        layerInd = reshape(round(mean(reshape(tmpSurfInds(1:numSurfPicks,1:bedPickTracesPerFile),...
                            numSurfPicks,bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),2)),numSurfPicks,bedPickTracesPerFile/bedPickDecimate);
                    elseif (sigOpt ==2)
                        Data = tmpecho(1:echoLen,1:bedPickDecimate:bedPickTracesPerFile);
                        UTC_time = tmputc(1:bedPickDecimate:bedPickTracesPerFile);
                        Latitude = tmpLat(1:bedPickDecimate:bedPickTracesPerFile);
                        Longitude = tmpLon(1:bedPickDecimate:bedPickTracesPerFile);
                        Elevation = tmpFlightElev(1:bedPickDecimate:bedPickTracesPerFile);
                        layerInd = tmpSurfInds(:,1:bedPickDecimate:bedPickTracesPerFile);
                    elseif (sigOpt ==3)
                        Data = filter(bedpickFilt,1,tmpecho(1:echoLen,1:bedPickTracesPerFile),[],2);
                        Data = Data(:,1:bedPickDecimate:end);
                        UTC_time = tmputc(1:bedPickDecimate:bedPickTracesPerFile);
                        Latitude = tmpLat(1:bedPickDecimate:bedPickTracesPerFile);
                        Longitude = tmpLon(1:bedPickDecimate:bedPickTracesPerFile);
                        Elevation = tmpFlightElev(1:bedPickDecimate:bedPickTracesPerFile);
                        layerInd = tmpSurfInds(:,1:bedPickDecimate:bedPickTracesPerFile);
                    end
                    layerInd(isnan(layerInd)) = numel(Time);
                    Surface = Time(min(layerInd,[],1));
                    if iscolumn(Surface)
                        Surface = Surface';
                    end
                    Surface(Surface> Time(end-2)) = NaN;
                    if motionComp
                        Surf_Elev = (userRefAlt/3.28) - Surface*cAir/2;
                    else
                        Surf_Elev = Elevation - Surface*cAir/2;
                    end
                    
                    numRemainingTraces = numRemainingTraces - bedPickTracesPerFile;
                    
                    tmpecho(1:echoLen,1:numRemainingTraces) = tmpecho(1:echoLen,bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmputc(1:numRemainingTraces) = tmputc(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpLat(1:numRemainingTraces) = tmpLat(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpLon(1:numRemainingTraces) = tmpLon(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpFlightElev(1:numRemainingTraces) = tmpFlightElev(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpSurfInds(:,1:numRemainingTraces) = tmpSurfInds(:,bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    outfileCount = outfileCount + 1;
                                       
                    if old_fileformat
                        save([fftDecimatedLineDataPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(UTC_time(1)),...
                            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,prodLevel.name,fileFieldSep,Sensor,fileFieldSep,num2str(outfileCount)],...
                            'Data','UTC_time','Time','Campaign','FlightNo','numAvg','Latitude','Longitude','Elevation','layerInd','Surface','Surf_Elev','prodLevel');
                    else
                        save([fftDecimatedLineDataPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(UTC_time(1)),...
                            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,prodLevel.name,fileFieldSep,Sensor,fileFieldSep,num2str(outfileCount)],...
                            'Data','UTC_time','Time','Campaign','FlightNo','numAvg','Latitude','Longitude','Elevation','layerInd','Surface','Surf_Elev','prodLevel');
                    end
                    
                    if ~configFileCopied
                        [~,configFilename,~] = fileparts(configScript);
                        copyfile([configFilename,'.mat'],[fftDecimatedLineDataPath,configFilename,'.mat'])
                        configFileCopied = true;
                    end
                end
            end
            if(numRemainingTraces > prfPreSar*60) % if there is more than a minute of data at the end of the batch save it
                numRemainingTraces = numRemainingTraces - mod(numRemainingTraces,bedPickDecimate);
                if(sigOpt ==1)
                    Data = reshape(mean(reshape(tmpecho(1:echoLen,1:numRemainingTraces),...
                        echoLen,bedPickDecimate,numRemainingTraces/bedPickDecimate),2),...
                        echoLen,numRemainingTraces/bedPickDecimate);
                    UTC_time = mean(reshape(tmputc(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    Latitude = mean(reshape(tmpLat(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    Longitude = mean(reshape(tmpLon(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    Elevation = mean(reshape(tmpFlightElev(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    layerInd = reshape(round(mean(reshape(tmpSurfInds(1:numSurfPicks,1:numRemainingTraces),...
                        numSurfPicks,bedPickDecimate,numRemainingTraces/bedPickDecimate),2)),numSurfPicks,numRemainingTraces/bedPickDecimate);
                elseif (sigOpt ==2)
                    Data = tmpecho(1:echoLen,1:bedPickDecimate:numRemainingTraces);
                    UTC_time = tmputc(1:bedPickDecimate:numRemainingTraces);
                    Latitude = tmpLat(1:bedPickDecimate:numRemainingTraces);
                    Longitude = tmpLon(1:bedPickDecimate:numRemainingTraces);
                    Elevation = tmpFlightElev(1:bedPickDecimate:numRemainingTraces);
                    layerInd = tmpSurfInds(:,1:bedPickDecimate:numRemainingTraces);
                elseif (sigOpt ==3)
                    Data = filter(bedpickFilt,1,tmpecho(1:echoLen,1:numRemainingTraces),[],2);
                    Data = Data(:,1:bedPickDecimate:end);
                    UTC_time = tmputc(1:bedPickDecimate:numRemainingTraces);
                    Latitude = tmpLat(1:bedPickDecimate:numRemainingTraces);
                    Longitude = tmpLon(1:bedPickDecimate:numRemainingTraces);
                    Elevation = tmpFlightElev(1:bedPickDecimate:numRemainingTraces);
                    layerInd = tmpSurfInds(:,1:bedPickDecimate:numRemainingTraces);
                end
                layerInd(isnan(layerInd)) = numel(Time);
                Surface = Time(min(layerInd,[],1));
                if iscolumn(Surface)
                    Surface = Surface';
                end
                Surface(Surface> Time(end-2)) = NaN;
                if motionComp
                    Surf_Elev = (userRefAlt/3.28) - Surface*cAir/2;
                else
                    Surf_Elev = Elevation - Surface*cAir/2;
                end
                
                outfileCount = outfileCount + 1;
                if old_fileformat
                    save([fftDecimatedLineDataPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(UTC_time(1)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,prodLevel.name,fileFieldSep,Sensor,fileFieldSep,num2str(outfileCount)],...
                        'Data','UTC_time','Time','Campaign','FlightNo','numAvg','Latitude','Longitude','Elevation','layerInd','Surface','Surf_Elev','prodLevel');
                else
                    save([fftDecimatedLineDataPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(UTC_time(1)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,prodLevel.name,fileFieldSep,Sensor,fileFieldSep,num2str(outfileCount)],...
                        'Data','UTC_time','Time','Campaign','FlightNo','numAvg','Latitude','Longitude','Elevation','layerInd','Surface','Surf_Elev','prodLevel');
                end
                
                if ~configFileCopied
                        [~,configFilename,~] = fileparts(configScript);
                        copyfile([configFilename,'.mat'],[fftDecimatedLineDataPath,configFilename,'.mat'])
                        configFileCopied = true;
                    end
            end
        end
        warning('on','all')
        diary off;
        if(configFileCopied)     % if yes, then the line was most likely processed
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'decimate',datestr(bedPickDecimatedTime,'_yyyymmdd_hhMMss'),logFileExt]);
        
        [~,configFileNameOnly,~] = fileparts(configScript);
        copyfile(configScript,[processLogPath,configFileNameOnly,fileFieldSep,'decimate',datestr(bedPickDecimatedTime,'_yyyymmdd_hhMMss'),'.m.log']);
        else % delete existing diary and start a fresh one
        [~,~,logext] = fileparts(logFile);
        if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
            delete(logFile);
        end
        end
        diary(logFile);  
    end
    if localDiary
        if strcmpi(get(0,'Diary'),'on')  %% if diary is still on - delete it
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
    fprintf('sirDecimateLevel1: Done!\n')
    toc
catch errMsg
    fprintf('sirDecimateLevel1: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('sirDecimateLevel1: Halting for debugging\n')
        keyboard
    end
end
end