function [] = sirLineBreaker(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the Breaks already processed Files by Lines and renames
% the files appropriately.

try
    fprintf('************** sirLineBreaker.m ****************\n')
    
    if nargin <1
        configScript = 'sirProcessConfig.m';
        logFile = 'sirProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('sirLineBreaker:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    elseif nargin <2
        configScript = newConfig;
        logFile = 'sirProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('sirLineBreaker:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    else
        configScript = newConfig;
        localDiary = false;
    end
    if ispc
        if(~isWindowsAdmin)
            warning('sirLineBreaker:noAdminPrevilage',['This process requires admin previlage\n'...
                ,'Please run matlab as administrator.\n This step will be skipped']);
            warning('off','sirLineBreaker:noAdminPrevilage');
            return;
        end
    end
    run(configScript);
    configParamsCurr = configParams;
    tic
    
    for ll = 1:numel(LineNum)   %% process each line separately
        startTime = LineTime(ll,1);
        stopTime = LineTime(ll,2);
        fftProcessedLineDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,LineNum{ll},filesep,dataDir,filesep];
        fftProcessedDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,Flight{ll},filesep,dataDir,filesep];
        processLogPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,LineNum{ll},filesep,logDir,filesep];
        if exist(fftProcessedDataPath,'dir')==0
            warning('sirLineBreaker:noLevel1Dir','Level1 Flight %s Directory does not exists!',Flight{ll});
            continue;
        end
        
        %% Check input directory and sort data files
        level1Files=dir([fftProcessedDataPath,['*',Flight{ll},'*.mat']]);
        numfiles=numel(level1Files);
        if(numfiles == 0)
            warning(['sirLineBreaker:','no',Flight{ll},'Files'],'No level1 %s files found!\n Moving on!\n',Flight{ll})
            continue;
        else
            for ii = 1:numfiles
                fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
                try
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                    warning('sirLineBreaker:oldFileFormat','Files in old filename format. These file name will not be supported in futre versions');
                    warning('off','sirPickSurf:oldFileFormat');
                end
            end
            [~,sx] = sort([level1Files.utc]);
            level1Files = level1Files(sx);
        end
        
        startfile=find([level1Files.utc]<startTime,1,'last');
        endfile=find([level1Files.utc]>stopTime,1,'first');
        
        % if end file is empty - it might mean this line goes beyond the
        % last file
        if isempty(endfile)
            endfile=numel(level1Files);
        end
        if isempty(startfile)
            startfile=1;
        end
        % check for the case when there is no data available for the given
        % line but there is data before the line time starts, in this case the end file and start file would be the last
        % file in the flight
        if ~isempty(startfile)
            if (endfile - startfile < 1)
                warning('sirLineBreaker:NoFilesSelected','No Files found for Line %s! Make sure Level1 data exists for this line',LineNum{ll})
                continue;
            end
        end
        
        %% Check for existing output directories and data (if any)
        if exist(fftProcessedLineDataPath,'dir')==0
            mkdir(fftProcessedLineDataPath)
        else
            dataFlist = dir([fftProcessedLineDataPath,'*',LineNum{ll},'*.mat']);
            if numel(dataFlist)>0
                fprintf('sirLineBreaker: Found existing data files\n');
                pcFlist = dir([fftProcessedLineDataPath,'*',configFnameRoot,'*.mat']);
                if numel(pcFlist)>0
                    [~,sx] = sort([pcFlist.datenum]);
                    pcFlist = pcFlist(sx);
                    fprintf('sirLineBreaker: Checking existing configuration data...\n')
                    load([fftProcessedLineDataPath,pcFlist(end).name],'configParams');
                    if isequal(configParamsCurr,configParams)
                        if ignoreConfigMatch && overwriteoutput
                            warning('sirLineBreaker:FlightDirExists',['All existing files in %s will be deleted!\n',...
                                'If this is not intended, press Ctrl + C to cancel'],LineNum{ll});
                            
                            fprintf('sirLineBreaker: Files will be deleted in %d seconds',9);
                            for ss = 9:-1:0
                                fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                                pause(1);
                            end
                            fprintf('\nsirLineBreaker: Deleted!\n')
                            delete([fftProcessedLineDataPath,'*',LineNum{ll},'*.mat']);
                            delete([fftProcessedLineDataPath,'*onfig*.mat']);
                        else
                            fprintf(['sirLineBreaker: Current settings are identical to the one %s processed data used.\n\t',...
                                'Existing %s data will not be re-processed.\n\tTo force processing, Remove configuration mat file from\n\t%s\n'],LineNum{ll},LineNum{ll},fftProcessedLineDataPath);
                        end
                    else
                        if overwriteoutput == 1
                            warning('sirLineBreaker:FlightDirExists',['All existing files in %s will be deleted!\n',...
                                'If this is not intended, press Ctrl + C to cancel'],LineNum{ll});
                            
                            fprintf('sirLineBreaker: Files will be deleted in %d seconds',9);
                            for ss = 9:-1:0
                                fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                                pause(1);
                            end
                            fprintf('\nsirLineBreaker: Deleted!\n')
                            delete([fftProcessedLineDataPath,'*',LineNum{ll},'*.mat']);
                            delete([fftProcessedLineDataPath,'*onfig*.mat']);
                        else
                            warning('sirLineBreaker:LineDirExists','Line %s Directory already exists - Line will not be processed!\n Set "overwriteoutput = 1" to force processing',LineNum{ll});
                            continue;
                        end
                    end
                else    %if for any reason there is no configuration file
                    if overwriteoutput == 1
                        warning('sirLineBreaker:FlightDirExists',['All existing files in %s will be deleted!\n',...
                            'If this is a mistake, press Ctrl + C to cancel'],LineNum{ll});
                        
                        fprintf('sirLineBreaker: Files will be deleted in %d seconds',9);
                        for ss = 9:-1:0
                            fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                            pause(1);
                        end
                        fprintf('\nsirLineBreaker: Deleted!\n')
                        delete([fftProcessedLineDataPath,'*',LineNum{ll},'*.mat']);
                        delete([fftProcessedLineDataPath,'*onfig*.mat']);
                    else
                        warning('sirLineBreaker:LineDirExists','Line %s Directory already exists - Line will not be processed!\n Set "overwriteoutput = 1" to force processing',LineNum{ll});
                        continue;
                    end
                end
            end
        end
        
        [~,configFilename,~] = fileparts(configScript);
        copyfile([configFilename,'.mat'],[fftProcessedLineDataPath,configFilename,'.mat'])
        
%% Start Breaking        
        fprintf('sirLineBreaker: Processing Line %s\n',LineNum{ll});
        for ii = startfile:endfile
            fprintf('sirLineBreaker: Copying file %d of %d\n',ii-startfile+1,endfile-startfile +1);
            fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
            sirLineFileName = [Project,fileFieldSep,LineNum{ll},fileFieldSep,fileNameFields{dateField},fileFieldSep,...
                fileNameFields{timeField},fileFieldSep,fileNameFields{LevelField},fileFieldSep,Sensor,'.mat'];
            if(exist([fftProcessedLineDataPath,filesep,sirLineFileName],'file') == 0)
                if ispc
                    status = system(['mklink /H ',[fftProcessedLineDataPath,filesep,sirLineFileName],' ',[fftProcessedDataPath,filesep,level1Files(ii).name]]);
                else
                    status = system(['ln ', [fftProcessedDataPath,filesep,level1Files(ii).name],' ',[fftProcessedLineDataPath,filesep,sirLineFileName], ' < /dev/null']);
                end
                if status ~= 0
                    warning('sirLineBreaker:NoFilesSelected','Failed to create Link to file %s',[fftProcessedDataPath,filesep,level1Files(ii).name])
                end
            else
                fprintf('File Exists. Skipped!\n');
            end
        end
        diary off;
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
        diary(logFile);
    end
    if localDiary
        if strcmpi(get(0,'Diary'),'on')  %% if diary is still on - delete it
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
    fprintf('sirLineBreaker: Done!\n');
    toc
catch errMsg
    fprintf('sirLineBreaker: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('sirLineBreaker: Halting for debugging\n')
        keyboard
    end
end
end