function [] = sirPickSurf(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the geoLocated data and processed further to get it ready



try
    fprintf('************** sirPickSurf.m ****************\n')
    if nargin <1
        configScript = 'sirProcessConfig.m';
        logFile = 'sirProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('sirPickSurf:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    elseif nargin <2
        configScript = newConfig;
        logFile = 'sirProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('sirPickSurf:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    else
        configScript = newConfig;
        localDiary = false;
    end
    run(configScript);
    configParamsCurr = configParams;
    tic
    
    maxnumTraces = ceil(maxLinesperBatch/numAvg);
    tempInd = zeros(numSurfPicks,maxnumTraces+surfFilterWin*4);
    % tmptxon = false(1,maxnumTraces);
    tmpecho = zeros(numel(Range),maxnumTraces*3);
    
    if exist('surfacePickWindowType','var')
        if strcmpi(surfacePickWindowType,'adaptive')
            fixedInds = false;
        else
            fixedInds = true;
        end
        fprintf('sirPickSurf: Using %s window for picking surface ...\n',surfacePickWindowType)
    else
        fixedInds = true;
        fprintf('sirPickSurf: Using a fixed window for picking surface ...\n')
    end
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %% Load all the files in the level1 directory
    for ff = 1:numel(uniqueFlightNum)
        %% Check files and directories:
        FlightNo = uniqueFlight{ff};
        surfPickedTime = now;
        configFileCopied = false;
        fftProcessedDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];
        processLogPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,logDir,filesep];
        if exist(fftProcessedDataPath,'dir')==0
            warning('sirPickSurf:noLevel1Dir','Level1 Flight %s Directory does not exists!',FlightNo);
            continue;
        end
        
        pcFlist = dir([fftProcessedDataPath,'*',configFnameRoot,'*.mat']);
        if numel(pcFlist)>0
            [~,sx] = sort([pcFlist.datenum]);
            pcFlist = pcFlist(sx);
            fprintf('sirPickSurf: Checking existing configuration data...\n')
            load([pcFlist(end).name],'configParams');
            if isequal(configParamsCurr.pickSurf,configParams.pickSurf)
                configmatch = true;
            else
                configmatch = false;
            end
        end
        
        fprintf('sirPickSurf: Processing Flight %s\n',FlightNo)
        
        %
        tmpFiles = dir([fftProcessedDataPath,'*',FlightNo,'*.mat',tmpExt]);
        if ~isempty(tmpFiles)
            fprintf('sirPickSurf: Recovering corrupted mat files from previous run...\n');
            for tt = 1:numel(tmpFiles)
                copyfile([fftProcessedDataPath,tmpFiles(tt).name],[fftProcessedDataPath,tmpFiles(tt).name(1:end-numel(tmpExt))])
                delete([fftProcessedDataPath,tmpFiles(tt).name]);
            end
        end
        level1Files=dir([fftProcessedDataPath,['*',FlightNo,'*.mat']]);
        numfiles=numel(level1Files);
        if(numfiles == 0)
            warning(['sirPickSurf:','no',FlightNo,'Files'],'No level1 %s files found!\n Moving on!\n',FlightNo)
            continue;
        else
            for ii = 1:numfiles
                fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
                try
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                    warning('sirPickSurf:oldFileFormat','Files in old filename format. Re-run diceProcessConfig to rename the files.');
                    warning('off','sirPickSurf:oldFileFormat');
                end
            end
            [~,sx] = sort([level1Files.utc]);
            level1Files = level1Files(sx);
        end
        %
        %% Find the surface based by looking for maximum power
        % The radar surface is spiky so it need to be smoothed. To mitigate the
        % edge effects of smoothing is done by loading 3 files at a time
        fprintf('sirPickSurf: Picking Layers from Radar ... \n');
        %
        refsurfind = initSurfInd;
        gotsurf = false;
        numfiles=numel(level1Files);
        fTime = [level1Files.utc];
        dTime = diff(fTime);
        batchind = find(dTime >filesperbatch*filePeriod*1.1);
        numbatches = numel(batchind)+1;
        batchmin(1) = 1;
        if isempty(batchind)
            batchmax(1) = numfiles;
        else
            batchmax(1) = batchind(1);
            batchmin(numbatches) = batchind(end)+1;
            batchmax(numbatches) = numfiles;
        end
        for bb=2:numbatches-1
            batchmin(bb) = batchind(bb-1)+1;
            batchmax(bb) = batchind(bb);
        end
        if numbatches>1
            warning('sirPickSurf:FilesNotContiguous','Files are not contiguous!');
        end
        
        minfNum=batchmin;
        maxfNum=batchmax;
        
        for batch=1:numbatches
            thisFileAlreadyPicked = false;
            gotsurfIndRange = false;
            if(maxfNum(batch)-minfNum(batch) +1 >=3)
                varlist = whos('-file',[fftProcessedDataPath,level1Files(ii).name]);
                thisFileAlreadyPicked = all(ismember({'layerInd'},{varlist.name}));
                % pre-load the first file
                clear txOn;
                load([fftProcessedDataPath,level1Files(minfNum(batch)).name],'radecho','Range','txOn','prodLevel')
                
                % backward compatibility
                if ~iscell(prodLevel.desc)
                    oldProdLevel = prodLevel;
                    clear prodLevel;
                    prodLevel.name = oldProLevel.name;
                    prodLevel.desc = {oldProdLevel.desc};
                end
                if ~ismember('time',fieldnames(prodLevel))
                    for pp = 1:numel(prodLevel.desc)
                        prodLevel.time{pp} = '?';
                    end
                end
                
                filesize1 = size(radecho,2);
                fileInds1 = 1:filesize1;
                tmpecho(:,fileInds1) = radecho;
                
                prodLevel1 = prodLevel;
                if ~exist('txOn','var')
                    txOn = ones(1,filesize1);
                end
                txOn1 = txOn;
                clear txOn;
                % pre-load the second file of the batch
                load([fftProcessedDataPath,level1Files(minfNum(batch)+1).name],'radecho','Range','txOn','prodLevel')
                
                % backward compatibility
                if ~iscell(prodLevel.desc)
                    oldProdLevel = prodLevel;
                    clear prodLevel;
                    prodLevel.name = oldProLevel.name;
                    prodLevel.desc = {oldProdLevel.desc};
                end
                if ~ismember('time',fieldnames(prodLevel))
                    for pp = 1:numel(prodLevel.desc)
                        prodLevel.time{pp} = '?';
                    end
                end
                
                filesize2 = size(radecho,2);
                fileInds2 = (filesize1+1):(filesize1+filesize2);
                tmpecho(:,fileInds2) = radecho;
                
                prodLevel2 = prodLevel;
                
                if ~exist('txOn','var')
                    txOn = ones(1,filesize2);               
                end
                txOn2 = txOn;
                clear txOn;
                fprintf('sirPickSurf: Processing %s File %d of %d\n',FlightNo,minfNum(batch),numfiles);
                
                if ~fixedInds
                    gotsurfIndRange = false;
                    [~,tmpSurfInds] = findpeaksMat(20*log10(abs(tmpecho(:,1:floor((filesize1+surfFilterWin*2)*0.05):(filesize1+surfFilterWin*2)))),1,surfPickOption,surfThresh);
                    if numel(find(~isnan(tmpSurfInds)))/numel(tmpSurfInds)>0.4 % 40% of the echoes returned surface indices
                        gotsurfIndRange = true;
                        surfWinInd = [max(2,nanmin(tmpSurfInds) - ceil(surfaceTrackingIndWin/2)) min(numel(Range),nanmax(tmpSurfInds) + ceil(surfaceTrackingIndWin/2))];
                    end
                end
                
                if fixedInds || ~gotsurfIndRange
                    if ~gotsurf % this is not a typo, it is indeed gotsurf.
                        surfWinInd = [max(2,refsurfind - ceil(surfaceAcqIndWin/2)),min(numel(Range),refsurfind + ceil(surfaceAcqIndWin/2))];
                    else
                        surfWinInd = [max(2,refsurfind - ceil(surfaceTrackingIndWin/2)),min(numel(Range),refsurfind + ceil(surfaceTrackingIndWin/2))];
                    end
                end
                
                [~,tempInd(:,1:filesize1+surfFilterWin*2)] = findpeaksMat(intuitive_lowpass2(20*log10(abs(tmpecho(surfWinInd(1):surfWinInd(2),1:(filesize1+surfFilterWin*2)))),surfFilterWin,1),numSurfPicks,surfPickOption,surfThresh);
                layerInd = surfWinInd(1) + tempInd(:,1:filesize1) -1;
                layerInd(:,~txOn1) = noTxSurfInd;
                maxPwrInd = layerInd(1,:);
                % lets check and see if a surface is found..
                if any(isnan(maxPwrInd))
                    if all(isnan(maxPwrInd))
                        warning('sirPickSurf:noSurfFound',['No surfaces above specified threshold\n',...
                            'File %s'],level1Files(minfNum(batch)).name);
                    end
                    maxPwrInd(isnan(maxPwrInd)) = 1;
                    planeHeight = Range(maxPwrInd)';
                    planeHeight(maxPwrInd == 1) = NaN;
                else
                    planeHeight = Range(maxPwrInd)';
                end
                
                %update the refsurfIndRad
                tmpGoodSurf = maxPwrInd(maxPwrInd>2);
                if ~isempty(tmpGoodSurf)
                    tmpGoodSurf = median(tmpGoodSurf(max(floor(numel(tmpGoodSurf)*0.9),1):end));
                end
                if ~isnan(tmpGoodSurf)
                    refsurfind = tmpGoodSurf;
                    gotsurf = true;
                else
                    gotsurf = false;
                end
                
                % file is only saved if it hasn't been picked or
                % if overwrite is enabled when: there is not config
                % match or configmatch is ignored
                if (~thisFileAlreadyPicked || ((~configmatch || ignoreConfigMatch) && overwriteoutput))
                    if save_append_backup
                        copyfile([fftProcessedDataPath,level1Files(minfNum(batch)).name],[fftProcessedDataPath,level1Files(minfNum(batch)).name,tmpExt]);
                    end
                    
                    prodLevel1.name = 'level1';
                    prodLevel1.desc{numel(prodLevel1.desc)+1} = 'surfPicked';
                    prodLevel1.time{numel(prodLevel1.time)+1} = datestr(surfPickedTime);
                    if any(ismember(prodLevel1.desc,'singleSurf'))
                        prodLevel1.desc{ismember(prodLevel1.desc,'singleSurf')} = 'singleSurfRem';
                    end
                    prodLevel = prodLevel1;
                    if ~configFileCopied
                        [~,configFilename,~] = fileparts(configScript);
                        copyfile([configFilename,'.mat'],[fftProcessedDataPath,configFilename,'.mat'])
                        configFileCopied = true;
                    end
                    
                    save([fftProcessedDataPath,level1Files(minfNum(batch)).name],'layerInd','prodLevel','-append');
                    
                    if save_append_backup
                        delete([fftProcessedDataPath,level1Files(minfNum(batch)).name,tmpExt]);
                    end
                else
                    fprintf('sirPickSurf: %d Skipped!\n',minfNum(batch));
                    if configmatch && ~ignoreConfigMatch
                        warning('sirPickSurf:conFigsMatch',['Existing surfaces use exact same pick configurations\n',...
                            'Set "ignoreConfigMatch = true" to reprocess these files']);
                        warning('off','sirPickSurf:conFigsMatch');
                    end
                    if ~overwriteoutput
                        warning('sirPickSurf:cannotOverWrite',['Existing surface Data cannot be overwritten\n',...
                            'Set "overwriteoutput = true" to reprocess these files']);
                        warning('off','sirPickSurf:cannotOverWrite');
                    end
                end
                
                for ii = minfNum(batch)+1:maxfNum(batch)-1
                    varlist = whos('-file',[fftProcessedDataPath,level1Files(ii).name]);
                    thisFileAlreadyPicked = all(ismember({'layerInd'},{varlist.name}));
                    %load the next file
                    clear txOn;
                    load([fftProcessedDataPath,level1Files(ii+1).name],'radecho','Range','txOn','prodLevel')
                    
                    % backward compatibility
                    if ~iscell(prodLevel.desc)
                        oldProdLevel = prodLevel;
                        clear prodLevel;
                        prodLevel.name = oldProLevel.name;
                        prodLevel.desc = {oldProdLevel.desc};
                    end
                    if ~ismember('time',fieldnames(prodLevel))
                    for pp = 1:numel(prodLevel.desc)
                        prodLevel.time{pp} = '?';
                    end
                    end
                    filesize3 = size(radecho,2);
                    % recompute inds for new iteration
                    fileInds3 = (filesize1 + filesize2 + 1): (filesize1+filesize2+filesize3);
                    tmpecho(:,fileInds3) = radecho;
                    
                    prodLevel3 = prodLevel;
                    if ~exist('txOn','var')
txOn = ones(1,filesize3);                    end
                    txOn3 = txOn;
                    clear txOn;
                    % findpeaksMat picks nSurfPick surfaces with selected options
                    fprintf('sirPickSurf: Processing %s File %d of %d\n',FlightNo,ii,numfiles);
                    
                    if thisFileAlreadyPicked && configmatch
                        if(~forceSurfPick )
                            warning('sirPickSurf:alreadySurfPicked',['One or more files already have surfaces\n',...
                                'Set "forceSurfPick = true" to reprocess these files']);
                            warning('off','sirPickSurf:alreadySurfPicked');
                            tmpecho(:,1:filesize2) = tmpecho(:,fileInds2);
                            tmpecho(:,(filesize2+1):(filesize2+filesize3)) = tmpecho(:,fileInds3);
                            fileInds1 = 1:filesize2;
                            fileInds2 = (filesize2+1):(filesize2+filesize3);
                            filesize1 = filesize2;
                            filesize2 = filesize3;
                            txOn1 = txOn2;
                            txOn2 = txOn3;
                            prodLevel2 = prodLevel3;
                            fprintf('sirPickSurf: %d Skipped!\n',ii);
                            continue;
                        end
                    end
                    
                    if ~fixedInds
                        gotsurfIndRange = false;
                        [~,tmpSurfInds] = findpeaksMat(20*log10(abs(tmpecho(:,1:floor((filesize1+surfFilterWin*2)*0.05):(filesize1+surfFilterWin*2)))),1,surfPickOption,surfThresh);
                        if numel(find(~isnan(tmpSurfInds)))/numel(tmpSurfInds)>0.4 % 40% of the echoes returned surface indices
                            gotsurfIndRange = true;
                            surfWinInd = [max(2,nanmin(tmpSurfInds) - ceil(surfaceTrackingIndWin/2)) min(numel(Range),nanmax(tmpSurfInds) + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                    if fixedInds || ~gotsurfIndRange
                        if ~gotsurf % this is not a typo, it is indeed gotsurf.
                            surfWinInd = [max(2,refsurfind - ceil(surfaceAcqIndWin/2)),min(numel(Range),refsurfind + ceil(surfaceAcqIndWin/2))];
                        else
                            surfWinInd = [max(2,refsurfind - ceil(surfaceTrackingIndWin/2)),min(numel(Range),refsurfind + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                    [~,tempInd(:,1:filesize2+surfFilterWin*4)] = ...
                        findpeaksMat(intuitive_lowpass2(20*log10(abs(tmpecho(surfWinInd(1):surfWinInd(2),(fileInds1(end)-surfFilterWin*2+1):(fileInds2(end)+surfFilterWin*2)))),surfFilterWin,1),numSurfPicks,surfPickOption,surfThresh);
                    layerInd = surfWinInd(1) + tempInd(:,surfFilterWin*2+1:filesize2+surfFilterWin*2) -1;
                    layerInd(:,~txOn2) = noTxSurfInd;
                    maxPwrInd = layerInd(1,:);
                    % lets check and see if a surface is found..
                    if any(isnan(maxPwrInd))
                        if all(isnan(maxPwrInd))
                            warning('sirPickSurf:noSurfFound',['No surfaces above specified threshold\n',...
                                'File %s'],level1Files(ii).name);
                        end
                        maxPwrInd(isnan(maxPwrInd)) = 1;
                        planeHeight = Range(maxPwrInd)';
                        planeHeight(maxPwrInd == 1) = NaN;
                    else
                        planeHeight = Range(maxPwrInd)';
                    end
                    
                    %update the refsurfIndRad
                    tmpGoodSurf = maxPwrInd(maxPwrInd>2);
                    if ~isempty(tmpGoodSurf)
                        tmpGoodSurf = median(tmpGoodSurf(max(floor(numel(tmpGoodSurf)*0.9),1):end));
                    end
                    if ~isnan(tmpGoodSurf)
                        refsurfind = tmpGoodSurf;
                        gotsurf = true;
                    else
                        gotsurf = false;
                    end
                    
                    % file is only saved if it hasn't been picked or
                    % if overwrite is enabled when: there is not config
                    % match or configmatch is ignored
                    if(~thisFileAlreadyPicked || ((~configmatch || ignoreConfigMatch) && overwriteoutput))
                        
                        prodLevel1.name = 'level1';
                        prodLevel1.desc{numel(prodLevel1.desc)+1} = 'surfPicked';
                        prodLevel1.time{numel(prodLevel1.time)+1} = datestr(surfPickedTime);
                        if any(ismember(prodLevel1.desc,'singleSurf'))
                            prodLevel1.desc{ismember(prodLevel1.desc,'singleSurf')} = 'singleSurfRem';
                        end
                        prodLevel = prodLevel2;
                        if ~configFileCopied
                            [~,configFilename,~] = fileparts(configScript);
                            copyfile([configFilename,'.mat'],[fftProcessedDataPath,configFilename,'.mat'])
                            configFileCopied = true;
                        end
                        
                        if save_append_backup
                            copyfile([fftProcessedDataPath,level1Files(ii).name],[fftProcessedDataPath,level1Files(ii).name,tmpExt]);
                        end
                        save([fftProcessedDataPath,level1Files(ii).name],'layerInd','prodLevel','-append');
                        if save_append_backup
                            delete([fftProcessedDataPath,level1Files(ii).name,tmpExt]);
                        end
                    else
                        fprintf('sirPickSurf: %d Skipped!\n',ii);
                        if configmatch && ~ignoreConfigMatch
                            warning('sirPickSurf:conFigsMatch',['Existing surfaces use exact same pick configurations\n',...
                                'Set "ignoreConfigMatch = true" to reprocess these files']);
                            warning('off','sirPickSurf:conFigsMatch');
                        end
                        if ~overwriteoutput
                            warning('sirPickSurf:cannotOverWrite',['Existing surface Data cannot be overwritten\n',...
                                'Set "overwriteoutput = true" to reprocess these files']);
                            warning('off','sirPickSurf:cannotOverWrite');
                        end
                    end
                    
                    tmpecho(:,1:filesize2) = tmpecho(:,fileInds2);
                    tmpecho(:,(filesize2+1):(filesize2+filesize3)) = tmpecho(:,fileInds3);
                    fileInds1 = 1:filesize2;
                    fileInds2 = (filesize2+1):(filesize2+filesize3);
                    filesize1 = filesize2;
                    filesize2 = filesize3;
                    txOn1 = txOn2;
                    txOn2 = txOn3;
                    prodLevel2 = prodLevel3;
                end
                varlist = whos('-file',[fftProcessedDataPath,level1Files(ii+1).name]);
                thisFileAlreadyPicked = all(ismember({'layerInd'},{varlist.name}));
                
                %process the last file in the batch
                fprintf('sirPickSurf: Processing %s File %d of %d\n',FlightNo,ii+1,numfiles);
                
                if (~thisFileAlreadyPicked || ((~configmatch || ignoreConfigMatch) && overwriteoutput))
                    
                    if ~fixedInds
                        gotsurfIndRange = false;
                        [~,tmpSurfInds] = findpeaksMat(20*log10(abs(tmpecho(:,(fileInds1(end)-surfFilterWin*2+1):floor(((fileInds2(end)+surfFilterWin*2)-(fileInds1(end)-surfFilterWin*2+1)+1)*0.05):(fileInds2(end)+surfFilterWin*2)))),1,surfPickOption,surfThresh);
                        if numel(find(~isnan(tmpSurfInds)))/numel(tmpSurfInds)>0.4 % 40% of the echoes returned surface indices
                            gotsurfIndRange = true;
                            surfWinInd = [max(2,nanmin(tmpSurfInds) - ceil(surfaceTrackingIndWin/2)) min(numel(Range),nanmax(tmpSurfInds) + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                    if fixedInds || ~gotsurfIndRange
                        if ~gotsurf
                            surfWinInd = [max(2,refsurfind - ceil(surfaceAcqIndWin/2)),min(numel(Range),refsurfind + ceil(surfaceAcqIndWin/2))];
                        else
                            surfWinInd = [max(2,refsurfind - ceil(surfaceTrackingIndWin/2)),min(numel(Range),refsurfind + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                    [~,tempInd(:,1:filesize2+surfFilterWin*4)] = ...
                        findpeaksMat(intuitive_lowpass2(20*log10(abs(tmpecho(surfWinInd(1):surfWinInd(2),(fileInds1(end)-surfFilterWin*2+1):(fileInds2(end)+surfFilterWin*2)))),surfFilterWin,1),numSurfPicks,surfPickOption,surfThresh);
                    layerInd = surfWinInd(1) + tempInd(:,surfFilterWin*2+1:filesize2+surfFilterWin*2) -1;
                    layerInd(:,~txOn3) = noTxSurfInd;
                    maxPwrInd = layerInd(1,:);
                    % lets check and see if a surface is found..
                    if any(isnan(maxPwrInd))
                        if all(isnan(maxPwrInd))
                            warning('sirPickSurf:noSurfFound',['No surfaces above specified threshold\n',...
                                'File %s'],level1Files(ii).name);
                        end
                        maxPwrInd(isnan(maxPwrInd)) = 1;
                        planeHeight = Range(maxPwrInd)';
                        planeHeight(maxPwrInd == 1) = NaN;
                    else
                        planeHeight = Range(maxPwrInd)';
                    end
                    
                    %update the refsurfIndRad
                    tmpGoodSurf = maxPwrInd(maxPwrInd>2);
                    if ~isempty(tmpGoodSurf)
                        tmpGoodSurf = median(tmpGoodSurf(max(floor(numel(tmpGoodSurf)*0.9),1):end));
                    end
                    if ~isnan(tmpGoodSurf)
                        refsurfind = tmpGoodSurf;
                        gotsurf = true;
                    else
                        gotsurf = false;
                    end
                    
                    prodLevel2.name = 'level1';
                    prodLevel2.desc{numel(prodLevel2.desc)+1} = 'surfPicked';
                    prodLevel2.time{numel(prodLevel2.time)+1} = datestr(surfPickedTime);
                    if any(ismember(prodLevel2.desc,'singleSurf'))
                        prodLevel2.desc{ismember(prodLevel2.desc,'singleSurf')} = 'singleSurfRem';
                    end
                    prodLevel = prodLevel2;
                    
                    if ~configFileCopied
                        [~,configFilename,~] = fileparts(configScript);
                        copyfile([configFilename,'.mat'],[fftProcessedDataPath,configFilename,'.mat'])
                        configFileCopied = true;
                    end
                    
                    if save_append_backup
                        copyfile([fftProcessedDataPath,level1Files(ii+1).name],[fftProcessedDataPath,level1Files(ii+1).name,tmpExt]);
                    end
                    save([fftProcessedDataPath,level1Files(ii+1).name],'layerInd','prodLevel','-append');
                    if save_append_backup
                        delete([fftProcessedDataPath,level1Files(ii+1).name,tmpExt]);
                    end
                else
                    fprintf('sirPickSurf: %d Skipped!\n',ii+1);
                    if configmatch && ~ignoreConfigMatch
                        warning('sirPickSurf:conFigsMatch',['Existing surfaces use exact same pick configurations\n',...
                            'Set "ignoreConfigMatch = true" to reprocess these files']);
                        warning('off','sirPickSurf:conFigsMatch');
                    end
                    if ~overwriteoutput
                        warning('sirPickSurf:cannotOverWrite',['Existing surface Data cannot be overwritten\n',...
                            'Set "overwriteoutput = true" to reprocess these files']);
                        warning('off','sirPickSurf:cannotOverWrite');
                    end
                end
            else
                for ii = minfNum(batch):maxfNum(batch)
                    fprintf('sirPickSurf: Processing %s File %d of %d\n',FlightNo,ii,numfiles);
                    varlist = whos('-file',[fftProcessedDataPath,level1Files(ii).name]);
                    thisFileAlreadyPicked = all(ismember({'layerInd'},{varlist.name}));
                    
                    if thisFileAlreadyPicked && configmatch
                        if(~forceSurfPick)
                            warning('sirPickSurf:alreadySurfPicked',['One or more files already have surfaces\n',...
                                'Set "forceSurfPick = true" to reprocess these files']);
                            warning('off','sirPickSurf:alreadySurfPicked');
                            fprintf('sirpickSurf: %d Skipped!\n',ii);
                            continue;
                        end
                    end
                    clear txOn;
                    load([fftProcessedDataPath,level1Files(ii).name],'radecho','Range','txOn','prodLevel')
                    
                    % backward compatibility
                    if ~iscell(prodLevel.desc)
                        oldProdLevel = prodLevel;
                        clear prodLevel;
                        prodLevel.name = oldProLevel.name;
                        prodLevel.desc = {oldProdLevel.desc};
                    end
                    if ~ismember('time',fieldnames(prodLevel))
                    for pp = 1:numel(prodLevel.desc)
                        prodLevel.time{pp} = '?';
                    end
                    end
                
                    if ~exist('txOn','var')
                    txOn = ones(1,size(radecho,2));
                    end
                
                    if ~fixedInds
                        gotsurfIndRange = false;
                        [~,tmpSurfInds] = findpeaksMat(20*log10(abs(radecho(:,1:floor(size(radecho,2)*0.05):end))),1,surfPickOption,surfThresh);
                        if numel(find(~isnan(tmpSurfInds)))/numel(tmpSurfInds)>0.4 % 40% of the echoes returned surface indices
                            gotsurfIndRange = true;
                            surfWinInd = [max(2,nanmin(tmpSurfInds) - ceil(surfaceTrackingIndWin/2)) min(numel(Range),nanmax(tmpSurfInds) + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                    if fixedInds || ~gotsurfIndRange
                        if ~gotsurf
                            surfWinInd = [max(2,refsurfind - ceil(surfaceAcqIndWin/2)),min(numel(Range),refsurfind + ceil(surfaceAcqIndWin/2))];
                        else
                            surfWinInd = [max(2,refsurfind - ceil(surfaceTrackingIndWin/2)),min(numel(Range),refsurfind + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                    [~,layerInd] = findpeaksMat(intuitive_lowpass2(20*log10(abs(radecho(surfWinInd,:))),surfFilterWin,1),numSurfPicks,surfPickOption,surfThresh);
                    layerInd(:,:) = surfWinInd(1) + layerInd - 1;
                    layerInd(:,~txOn) = noTxSurfInd;
                    maxPwrInd = layerInd(1,:);
                    % lets check and see if a surface is found..
                    if any(isnan(maxPwrInd))
                        if all(isnan(maxPwrInd))
                            warning('sirPickSurf:noSurfFound',['No surfaces above specified threshold\n',...
                                'File %s'],level1Files(ii).name);
                        end
                        maxPwrInd(isnan(maxPwrInd)) = 1;
                        planeHeight = Range(maxPwrInd)';
                        planeHeight(maxPwrInd == 1) = NaN;
                    else
                        planeHeight = Range(maxPwrInd)';
                    end
                    
                    %update the refsurfIndRad
                    tmpGoodSurf = maxPwrInd(maxPwrInd>2);
                    if ~isempty(tmpGoodSurf)
                        tmpGoodSurf = median(tmpGoodSurf(max(floor(numel(tmpGoodSurf)*0.9),1):end));
                    end
                    if ~isnan(tmpGoodSurf)
                        refsurfind = tmpGoodSurf;
                        gotsurf = true;
                    else
                        gotsurf = false;
                    end
                    
                    if (~thisFileAlreadyPicked || ((~configmatch || ignoreConfigMatch) && overwriteoutput))
                        
                        prodLevel.name = 'level1';
                        prodLevel.desc{numel(prodLevel.desc)+1} = 'surfPicked';
                        prodLevel.time{numel(prodLevel.time)+1} = datestr(surfPickedTime);
                        if any(ismember(prodLevel.desc,'singleSurf'))
                            prodLevel.desc{ismember(prodLevel.desc,'singleSurf')} = 'singleSurfRem';
                        end
                        if ~configFileCopied
                            [~,configFilename,~] = fileparts(configScript);
                            copyfile([configFilename,'.mat'],[fftProcessedDataPath,configFilename,'.mat'])
                            configFileCopied = true;
                        end
                        
                        if save_append_backup
                            copyfile([fftProcessedDataPath,level1Files(ii).name],[fftProcessedDataPath,level1Files(ii).name,tmpExt]);
                        end
                        save([fftProcessedDataPath,level1Files(ii).name],'layerInd','prodLevel','-append');
                        if save_append_backup
                            delete([fftProcessedDataPath,level1Files(ii).name,tmpExt]);
                        end
                    else
                        fprintf('sirPickSurf: %d Skipped!\n',ii);
                        if configmatch && ~ignoreConfigMatch
                            warning('sirPickSurf:conFigsMatch',['Existing surfaces use exact same pick configurations\n',...
                                'Set "ignoreConfigMatch = true" to reprocess these files']);
                            warning('off','sirPickSurf:conFigsMatch');
                        end
                        if ~overwriteoutput
                            warning('sirPickSurf:cannotOverWrite',['Existing surface Data cannot be overwritten\n',...
                                'Set "overwriteoutput = true" to reprocess these files']);
                            warning('off','sirPickSurf:cannotOverWrite');
                        end
                    end
                end
            end
        end
        warning('on','all');
        diary off;
        
        if(configFileCopied)    % if true, then the flight was most likely processed
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            if ~exist(processLogPath,'dir')
                mkdir(processLogPath);
            end
            movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'surfpick',datestr(surfPickedTime,'_yyyymmdd_hhMMss'),logFileExt]);
            if runCount == 1
                [~,configFileNameOnly,~] = fileparts(configScript);
                copyfile(configScript,[processLogPath,configFileNameOnly,fileFieldSep,'surfpick',datestr(surfPickedTime,'_yyyymmdd_hhMMss'),'.m.log']);
            end
        else % delete existing diary and start a fresh one
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
        diary(logFile);
    end
    if localDiary
        if strcmpi(get(0,'Diary'),'on')  %% if diary is still on - delete it
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
    fprintf('sirPickSurf: Done!\n');
    toc
catch errMsg
    fprintf('sirPickSurf: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('sirPickSurf: Halting for debugging\n')
        keyboard
    end
end
end