function [] = sirProcess(configFile,logFile)
% This is the top level function for processing Shallow Ice Radar data. This script
% calls all functions required to process selected lines. All
% configurations for processing are set in "sirProcessConfig.m". In order to
% proces a new flight or line, change the settings in "sirProcessConf ig.m"

try
    fprintf('*--------------- sirProcess.m -----------------*\n')
    %% Through all scripts/functions
    if ~exist('logFile','var')
        logFile = 'sirProcessLog.log';
    end
    if exist(logFile,'file')
        warning('sirProcess:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    cleanLogs = onCleanup(@() cleanLogFileOnCrash(logFile));
    if ~exist('configFile','var')
        configFile = 'sirProcessConfig.m';
    end
    
    % check if fileBreaker is enabled, if yes then windows run requires admin
    % previlage
    if ispc
        needAdmin = false;
        fList = matlab.codetools.requiredFilesAndProducts('sirProcess.m','toponly');
        for ddn = 1:numel(fList)
            [~,filenameOnly,~] = fileparts(fList{ddn});
            if strcmp(filenameOnly,'sirLineBreaker')
                needAdmin = true;
                break;
            else
                needAdmin = false;
            end
        end
        if(needAdmin && ~ isWindowsAdmin)
            warning('sirProcess:noAdminPrevilage',['This process requires admin previlage\n'...
                ,'Please run matlab as an administrator.']);
            return;
        end
    end
    
    warning('on','all');
    
    % Data parsing, fft and motion compenstaion etc happens here
    sirFFTandMotionComp(configFile,logFile);
    
    % Pick the surface
    sirPickSurf(configFile,logFile);
    % Break processed Sir files by Lines - This creates
    sirLineBreaker(configFile,logFile);
    % Decimate for bed picking
    sirDecimateLevel1(configFile,logFile);
    
    if strcmpi(get(0,'Diary'),'on')  %% if diary is still on - delete it
        diary off;
        fprintf('sirProcess: Cleaning Up!\n');
        [~,~,logext] = fileparts(logFile);
        if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
            delete(logFile);
        end
    end
    
    fprintf('sirProcess: Done!\n');
    clear('cleanLogs');
catch errMsg
    % In case of error, display it and quit:
    fprintf('sirProcess: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    fprintf('sirProcess: Done!\n')
    if ~usejava('desktop')  % exit if running in command line
        exit;
    end
end