function [] = repairSirPicksSurfaceElev(rootDir)
% Parameters: rootDir the root directory of Level1a/level2a or level3 Sir
% or Dice data.
% This program methodically searches for all sir data files in all
% directories in the path and fixes the miscalculation in calculation of
% surface elevation.
% This program need RW access to the data rootDir.
% This program is set to fix all SIR picks run through the original script
% that was only meant for DICE and thus miscalculated SurfElev.
% Version 1.0.0

% Tej Dhakal (modified by Isabel Cordero)
% 05/03/2019
% LDEO, Columbia University

if nargin <1
    if usejava('desktop')
        rootDir = uigetdir('Select Data root directory to fix');
    else
        error('Requires path to Data as argument')
    end
end
logFile = ['fixSirPicksSurfElevLog',datestr(now,'_yyyymmdd_hhMMss'),'.log'];
diary(logFile);

% get a list of all files and directories in the path
% This includes present dir '.' and parent dir '..'
fprintf('Working in %s ..\n',rootDir);
backupFiles = dir([rootDir,filesep,'*fixSurfElev_bak']);
if ~isempty(backupFiles)
    fprintf('Found %d backup files\nRestoring..\n',numel(backupFiles));
    for bff = 1:numel(backupFiles)
        [~,filenameOnly,~] = fileparts(backupFiles(bff).name);
        movefile([rootDir,filesep,backupFiles(bff).name],[rootDir,filesep,filenameOnly,'.mat']);
        fprintf('%s\n',[filenameOnly,'.mat']);
    end
    fprintf('Done!\n');
end
fList = dir([rootDir,filesep,'*SIR*.mat']);     % Changed from DICE to SIR
% Fix the files in the current diretory
if ~isempty(fList)
    fixStatus = false(numel(fList),1);
  else
   fixStatus = false;
end
for ff = 1:numel(fList)
    fixStatus(ff) = repairSirSurfElev([rootDir,filesep,fList(ff).name]);
end
fprintf('Done!\n');
%stop the diary and move the current log file to the
diary off;

if any(fixStatus)
    [~,logFileNameOnly,logFileExtOnly] = fileparts(logFile);
    logOutDir = [rootDir,filesep,'..',filesep,'log',filesep];
    if ~exist(logOutDir,'dir')
        mkdir(logOutDir);
    end
    movefile(logFile,[logOutDir,filesep,logFileNameOnly,logFileExtOnly]);
else
    delete(logFile);
end
% work through all the subdirectories
dirList = dir(rootDir);
% exclude all files in the dir
dirList = dirList([dirList.isdir]);
% exclude current and last dir as well as some known directory names
dirList = dirList([~ismember({dirList.name},'..')]);
dirList = dirList([~ismember({dirList.name},'.')]);
dirList = dirList([~ismember({dirList.name},'log')]);
dirList = dirList([~ismember({dirList.name},'txt')]);
dirList = dirList([~ismember({dirList.name},'png')]);
dirList = dirList([~ismember({dirList.name},'pdf')]);
dirList = dirList([~ismember({dirList.name},'bin')]);
dirList = dirList([~ismember({dirList.name},'sir')]);
dirList = dirList([~ismember({dirList.name},'pnt')]);
dirList = dirList([~ismember({dirList.name},'gravity')]);
dirList = dirList([~ismember({dirList.name},'doc')]);
dirList = dirList([~ismember({dirList.name},'lidar')]);
dirList = dirList([~ismember({dirList.name},'magnetics')]);
dirList = dirList([~ismember({dirList.name},'optics')]);
dirList = dirList([~ismember({dirList.name},'auxiliary')]);

% recursively calls itself until no subdirectories available
for ddl = 1:numel(dirList)
    repairSirPicksSurfaceElev([rootDir,filesep,dirList(ddl).name]);
end
end
