% Surface Elevation Fixer
% This function replaces the erroneous Surf_Elev varaible in SIR picks

% Note: The Surface Elevation calcuation during decimation (erroneously)
% didn't take the transmit time into account. The original fix goes through
% each file and recomputes and replaces the existing Surf_Elev variable.
% That fix was specific to DICE files and was run on SIR picks. This code
% replaces the Surf Elev variables incorrectly fixed, with the variables in
% its decimated counterpart, repairing the "fix".
% This allows the code to be run on the same file more than once without
% causing further issues.
% Version 1.5.0

function [fixed] = repairSirSurfElev(inputfile)
fileFieldSep = '_';

if nargin<1
    if usejava('desktop')
        [file,path] = uigetfile('*.mat','Select a dice level1a or Picks file to fix');      % Changed from level2a to level1a
        inputfile = fullfile(path,file);
    else
        error('fixSirSurfElev:InvalidPath','Requires full level1a or Picks file as argument')
    end
end
fixed = false;
[~,filenameonly,~] = fileparts(inputfile);
PickedFile = filenameonly;
fileNameParts = strsplit(filenameonly,fileFieldSep);
if ~any([any(ismember(fileNameParts,'level1a')),any(ismember(fileNameParts,'Picks'))])      % Changed from level2a to level1a
    return;
end

decimatedDir = ('decimated');
matDir = ('mat');
lineNum = fileNameParts{2};
fileDate = fileNameParts{3};
fileTime = fileNameParts{4};
fileProdLevel = fileNameParts{5};

dataDir = ('D:\trunk\icepod\antarctica\20162017\radar\sir\fft\stack04\');
addpath(dataDir);
decimatedSirDir = dir([dataDir,lineNum,filesep,decimatedDir,filesep,matDir,filesep]);

while any(ismember(fileNameParts,'Picks'))
    fprintf('fixSirSurfElev: Working on %s to fix Surface Variables!\n', PickedFile);
    try
        for zz = 3:numel(decimatedSirDir)
            originalFileName = decimatedSirDir(zz).name;
            [~,ogfilenameonly,~] = fileparts(originalFileName);
            ogFileNameParts = strsplit(ogfilenameonly,fileFieldSep);

            if (~(any(ismember(fileDate,ogFileNameParts)) && ...
                    any(ismember(fileTime,ogFileNameParts)) && ...
                    any(ismember(fileProdLevel,ogFileNameParts))))
                continue;
            else
                addpath([dataDir,lineNum,filesep,decimatedDir,filesep,matDir,filesep]);
                fprintf('fixSirSurfElev: Using %s surface variables to fix %s surface variables!\n', originalFileName, PickedFile);
                load(originalFileName, 'Elevation', 'Surf_Elev', 'Surface', 'Time');
                origSurfElev = Surf_Elev;
                origSurfTime = Surface;
                origTime = Time;
                clear vars 'Surf_Elev', 'Time';
                
                load(inputfile, 'Surf_Elev', 'SurfTime', 'Time','userRefAlt');
                cAir = 299792458;
%                 compSurfElev = Elevation - origSurfTime*cAir/2;
                compSurfElev = (userRefAlt/3.28) - origSurfTime*cAir/2;


                Surf_Elev = compSurfElev;
                SurfTime = origSurfTime;
                Time = origTime;
                fixed = true;

                save(inputfile, 'Surf_Elev', 'SurfTime', 'Time', '-append');
                fprintf('fixSirSurfElev: Finished updating surface variables for %s!\n', PickedFile);
                return; 
            end
        end
    catch errorid
        disp('Caught an Error!!')
        disp(errorid.identifier)
        disp(errorid.message)
        for jj = 1:numel(errorid.stack)
            fprintf('In Function %s, Line %d\n',errorid.stack(jj).name,errorid.stack(jj).line)
        end
    warning('on','all');
    return;
    end
end
end
