%% FMCW Radar Simulation
clear all; clc; %close all;
%% Constants
c = 299792458;      % speed of light m/s
v_ice = 1.68e8;       % Refractive index of ice layer
n_ice = c/v_ice;
mult= 1e6;
t0 = 250e-6;            % chirp Period
%% Radar Parameters
%Range
z = [100];
h = 762;       % Range
R = h + z;
v = (h*c + z*v_ice)./(h + z);
Gt = 31.65; % 15 dBi
Gr = 31.65;
fm = 1/2./t0;           % Modulating Frequency
df = 600*mult;             % Chirp bandwidth
f_dot = df./t0;         % Chirp rate
df_per_m = 4*fm*df./v;  % Resolution- Frequency per m
fb = 4*R.*fm*df./v;     % Expected beat Frequency
f_min = 1700*mult;
f_max = f_min + f_dot*t0;
f0 = 2*mult*1e3;               % Tx signal center frequency
lambda = v./f0;         % average wavelength
%% Transmit Signal
fs = 5*mult*1e3;        % Sampling Frequency (Just for matlab)
Pt = 1;                 % Signal Power
phi = 0;                % Tx Initial phase 
tl = 0:1/fs:t0;         % signal Duration (rising chirp)
t = 0:1/fs:2*t0;        % total signal duration 
% Baseband
A = 1;
Tx_IF = A*cos(2*pi*(f_min + f_dot/2*tl).*tl + phi);
% Tx_IF = A/2*cos(2*pi*(f_min + f_dot/2*tl).*tl + phi)+A/2*sin(2*pi*(f_min
% + f_dot/2*tl).*tl + phi); Quadrature out
% Tx_IF = [Tx_IF wrev(Tx_IF(1:end-1))];      % This creates triangle
% Tx_IF = [Tx_IF Tx_IF Tx_IF Tx_IF];
Tx_IF = [Tx_IF];
L = length(Tx_IF);
NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
F_Tx_IF = 2*abs(fft(Tx_IF,NFFT))/L;
fif = fs/2*linspace(0,1,NFFT/2+1);
figure
plot(fif/mult,10*log10(F_Tx_IF(1:NFFT/2+1)))
title('Amplitude Spectrum of IF')
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on
axis([0 1.5*f_max/mult -80 0]);
% figure
% spectrogram(Tx_IF,256,128,256,fs,'yaxis')
% title('Spectrogram of IF')
% axis([0 8*t0 f_min f_max]);

%% Mixing and Filtering
% fc = 1.5*mult*1e3;
% LO = cos(2*pi*fc*t);
Tx_RF = Tx_IF;%.*LO;
% N   = 30;    % Order
% Fc1 = 1.6*mult*1e3;  % First Cutoff Frequency
% Fc2 = 2.4*mult*1e3;  % Second Cutoff Frequency

% Construct an FDESIGN object and call its BUTTER method.
% Fltr  = fdesign.bandpass('N,F3dB1,F3dB2', N, Fc1, Fc2, fs);
% Hd = design(Fltr, 'butter');

% Tx_RF = filter(Hd,Tx_RF);
% Tx_RF = Tx_RF/max(Tx_RF);
% F_Tx_RF = 2*abs(fft(Tx_RF,NFFT))/L;
% frf = fs/2*linspace(0,1,NFFT/2+1);
% Plot single-sided amplitude spectrum.
% figure
% plot(frf/mult,10*log10(F_Tx_RF(1:NFFT/2+1))) 
% title('Amplitude Spectrum of RF')
% xlabel('Frequency (MHz)')
% ylabel('|F_Tx_RF| (dB)')
% grid on
% % axis([1000 1.2*(f_max+fc)/mult -80 0]);
% figure
% spectrogram(Tx_RF,256,128,256,fs,'yaxis')
% title('Spectrogram of RF')
% axis([0  t fc+fmin fc+f_max]);
%% Ice Properties
L_ice = exp(z.*0.0020);     % absorption loss for ice
Tai = 0.83167; % Power Transmission Factor
Pr = ((lambda/4/pi).^2.*Gt.*Gr.*Tai.^2./(2.*(h+z./n_ice)).^2./L_ice.^2);
Pr = (Pr/max(Pr));
%% Rx Signal
t_delay = 2*R./v;      % time for signal to return
numpad = floor(t_delay.*fs);
Padded_TX = [Tx_RF zeros(1,max(numpad))];
Rx = zeros(1,length(Padded_TX));
%zero pad the signal in time domain to mimic Rx delay
for i = 1:length(numpad)
    Rx = Rx + Pr(i)*[zeros(1,numpad(i)) Tx_RF zeros(1, numpad(end)- numpad(i))];
end
%% Mixing and Filtering
frx = 50*mult;
Rx = Rx/max(Rx)+ randn(size(Rx))*3;
Fb_Rx = Padded_TX.*Rx;
Fb_Rx = downsample(Fb_Rx,fs/frx);
Fb_Rx = Fb_Rx.*hanning(length(Fb_Rx))';
fs = frx;
L = length(Fb_Rx);
NFFT = 2^nextpow2(16*L);
frf = fs/2*linspace(0,1,NFFT/2+1);
Range = Fb2Range(df,t0,frf);
% Fb_Rx = sirlpf(Fb_Rx);        
Fbeat = abs(fft(Fb_Rx,NFFT))/L;
% Plot single-sided amplitude spectrum.
figure
plot(frf/mult,20*log10(Fbeat(1:NFFT/2+1)))
title('Amplitude Spectrum of Beat Frequency')
xlabel('Frequency (MHz)')
ylabel('|Fb_Rx| (dB)')
grid on

figure
plot(Range,20*log10(Fbeat(1:NFFT/2+1)))
title('Amplitude Spectrum of Beat Frequency')
xlabel('Range, m)')
ylabel('Power (dB)')
grid on

% axis([0 30 -80 0]);
% figure
% spectrogram(Fb_Rx,256,32,256,fs,'yaxis')
% title('Spectrogram of Beat Frequency')
