%% LFMCW Optimization
clear all;
close all;
clc;
c = 299792458;      % speed of light m/s0
z = [0 300];
h = 1e3;           % Flight altitude, change this to plot diffent altitude
R = h + z;          % total range
n_ice = 1.785;       % refractive index of ice
v = (h*c/n_ice + z*c/n_ice)./(h + z); % average velocity through ice thickness
t = 100:100:10000;      % chirp period in us
t0 = t/1e6;         % chirp period in s
B = 200e6:100e6:8000e6;      % bandwidth Hz
delR = zeros(length(t0),length(B),length(R));   % Range resolution
Fbeat = zeros(length(t0),length(B),length(R));  % Beat Frequency
for i = 1:length(t0)
    for j = 1:length(B)
        for k = 1:length(R);
        delR(i,j,k) = v(k)^2*t0(i)/(2*B(j)*(v(k)*t0(i) -2*R(k)));  % Worst range resolution at surface
        Fbeat(i,j,k) = 2*R(k)*B(j)/v(k)/t0(i);                    % Maximum beat frequency at 300 m depth
        end
    end
end

AtR = 120*2*t0;    % Along-track resolution
BW = [200e6 600e6 4000e6];    % change this to plot different bandwidhs
figure
subplot 211
plot(t0/1e-6,delR(:,find(B==BW(1)),1),'r.',t0/1e-6,delR(:,find(B==BW(2)),1),'b.',t0/1e-6,delR(:,find(B==BW(3)),1),'k.')
title({'Range resolution vs chirp period @ ice surface'; [num2str(h) ' m flight altitude,' ]})
xlabel('Chirp period (\mus)');
ylabel('Range Resolution (m)')
legend(['BW = ' num2str(BW(1)/1e6) ' MHz'],['BW = ' num2str(BW(2)/1e6) ' MHz'],['BW = ' num2str(BW(3)/1e6) ' MHz'])
grid on;
subplot 212
plot(t0/1e-6,delR(:,find(B==BW(1)),2),'r.',t0/1e-6,delR(:,find(B==BW(2)),2),'b.',t0/1e-6,delR(:,find(B==BW(3)),2),'k.')
title({'Range resolution vs chirp period @ 300 m deep ice'; [num2str(h) ' m flight altitude,' ]})
xlabel('Chirp period (\mus)');
ylabel('Range Resolution (m)')
legend(['BW = ' num2str(BW(1)/1e6) ' MHz'],['BW = ' num2str(BW(2)/1e6) ' MHz'],['BW = ' num2str(BW(3)/1e6) ' MHz'])
grid on;

cp = [100 400 2000];    % change this to plot different chirip periods
figure
subplot 211
plot(B/1e6,delR(find(t==cp(1)),:,1),'r.',B/1e6,delR(find(t==cp(2)),:,1),'b.',B/1e6,delR(find(t==cp(3)),:,1),'k.')
title({'Range resolution vs Bandwidth @ ice surface'; [num2str(h) ' m flight altitude,' ]})
xlabel('Bandwidth MHz');
ylabel('Range Resolution (m)')
legend(['t0 = ' num2str(cp(1)) ' \mus'],['t0 = ' num2str(cp(2)) ' \mus'],['t0 = ' num2str(cp(3)) ' \mus'])
grid on;
subplot 212
plot(B/1e6,delR(find(t==cp(1)),:,2),'r.',B/1e6,delR(find(t==cp(2)),:,2),'b.',B/1e6,delR(find(t==cp(3)),:,2),'k.')
title({'Range resolution vs Bandwidth @ 300 m deep ice'; [num2str(h) ' m flight altitude,' ]})
xlabel('Bandwidth MHz');
ylabel('Range Resolution (m)')
legend(['t0 = ' num2str(cp(1)) ' \mus'],['t0 = ' num2str(cp(2)) ' \mus'],['t0 = ' num2str(cp(3)) ' \mus'])
grid on;

figure
plot(t,AtR(1,:),'.')
title('Along-track resolution vs chirp period')
xlabel('Chirp Period (\mus)')
ylabel('Along-track Resolution (m)')
grid on

figure
subplot 211
plot(t0/1e-6,Fbeat(:,find(B==BW(1)),1)/1e6,'r.',t0/1e-6,Fbeat(:,find(B==BW(2)),1)/1e6,'b.',t0/1e-6,Fbeat(:,find(B==BW(3)),1)/1e6,'k.')
title({'Beat Frequency vs chirp period @ ice surface'; [num2str(h) ' m flight altitude,' ]})
xlabel('Chirp period (\mus)');
ylabel('Beat Frequency (MHz)')
legend(['BW = ' num2str(BW(1)/1e6) ' MHz'],['BW = ' num2str(BW(2)/1e6) ' MHz'],['BW = ' num2str(BW(3)/1e6) ' MHz'])
grid on;
subplot 212
plot(t0/1e-6,Fbeat(:,find(B==BW(1)),2)/1e6,'r.',t0/1e-6,Fbeat(:,find(B==BW(2)),2)/1e6,'b.',t0/1e-6,Fbeat(:,find(B==BW(3)),2)/1e6,'k.')
title({'Beat Frequency vs chirp period @ 300 m deep ice'; [num2str(h) ' m flight altitude,' ]})
xlabel('Chirp period (\mus)');
ylabel('Beat Frequency (MHz)')
legend(['BW = ' num2str(BW(1)/1e6) ' MHz'],['BW = ' num2str(BW(2)/1e6) ' MHz'],['BW = ' num2str(BW(3)/1e6) ' MHz'])
grid on;

figure
subplot 211
plot(B/1e6,Fbeat(find(t==cp(1)),:,1)/1e6,'r.',B/1e6,Fbeat(find(t==cp(2)),:,1)/1e6,'b.',B/1e6,Fbeat(find(t==cp(3)),:,1)/1e6,'k.')
title({'Beat Frequency vs Bandwidth @ ice surface'; [num2str(h) ' m flight altitude,' ]})
xlabel('Bandwidth MHz');
ylabel('Beat Frequency (MHz)')
legend(['t0 = ' num2str(cp(1)) ' \mus'],['t0 = ' num2str(cp(2)) ' \mus'],['t0 = ' num2str(cp(3)) ' \mus'])
grid on;
subplot 212
plot(B/1e6,Fbeat(find(t==cp(1)),:,2)/1e6,'r.',B/1e6,Fbeat(find(t==cp(2)),:,2)/1e6,'b.',B/1e6,Fbeat(find(t==cp(3)),:,2)/1e6,'k.')
title({'Beat Frequency vs Bandwidth @ 100 m deep ice'; [num2str(h) ' m flight altitude,' ]})
xlabel('Bandwidth MHz');
ylabel('Beat Frequency (MHz)')
legend(['t0 = ' num2str(cp(1)) ' \mus'],['t0 = ' num2str(cp(2)) ' \mus'],['t0 = ' num2str(cp(3)) ' \mus'])
grid on;

%% parameter choice
cp = 250;      %  us
BW = 600e6;      % Hz
i = find(t==cp);
j = find(B == BW);

fprintf('Flight Altitude: %dm\n',h)
fprintf('Snow Depth(m)\t\t %d\t\t\t%d \n',z(1),z(2))
fprintf('Range Resolution(m)\t%0.4g\t\t%0.4g\n',delR(i,j,1),delR(i,j,2))
fprintf('Beat Frequency(m)\t%0.4g\t\t%0.4g\n',Fbeat(i,j,1)/1e6,Fbeat(i,j,2)/1e6)
fprintf('Along-track Resolution(m) %0.4g\n',AtR(i))
%% surface plots
% figure
% h = surf(B/1e6,t,delR(:,:,1))
% title({'Range Resolution @ ice Surface'; [num2str(h) ' m flight altitude,' ]})
% xlabel('Bandwidth (MHz)')
% ylabel('Chrip Period (\mus)')
% zlabel('Range Resolution (m)')
% set(h, 'edgecolor','none')
% figure
% h = surf(B/1e6,t,Fbeat(:,:,2)/1e6)
% title({'Beat Frequency @ 300 m deep ice'; [num2str(h) ' m flight altitude,' ]})
% xlabel('Bandwidth (MHz)')
% ylabel('Chrip Period (\mus)')
% zlabel('Beat Frequency (MHz)')
% set(h, 'edgecolor','none')