function [] = sirFindAndFixFiles(rootDir)
% Parameters: rootDir the root directory of sir Data.
% This program methodically searches for all sir data files in all
% directories in the path and fixes them if there are any issues with the
% data.
% This program need RW access to the data rootDir.
% Any previously fixed files are first restored to their original state
% Any broken/corrupt data files are saved with .sir-orig extension and an a
% new file with same name is created with fixed data.
% Tejendra Dhakal
% 05/06/2016
% LDEO, Columbia University

if nargin <1
    if usejava('desktop')
        rootDir = uigetdir('Select SIR directory to fix');
    else
        error('Requires path to sir data as argument')
    end
end
%% restore any previously fixed files
restoreOrigSirFiles(rootDir);
% get a list of all sir files and directories in path
% This includes present dir '.' and parent dir '..'
fprintf('Working in %s..\n',rootDir);
fList = dir([rootDir,filesep,'*.sir']);
for ff = 1:numel(fList)
    fixSirFile([rootDir,filesep,fList(ff).name]);
end

dirList = dir(rootDir);
dirList = dirList([dirList.isdir]);
dirList = dirList([~ismember({dirList.name},'..')]);
dirList = dirList([~ismember({dirList.name},'.')]);
% recursively go through all folders and subfolders
for ddl = 1:numel(dirList)
    sirFindAndFixFiles([rootDir,filesep,dirList(ddl).name]);
end
end
