% Tej Dhakal 06/15/2016
% LDEO, Columbia University
% This generates a text file with figure informatin for sir images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Important Note: The figure files are NOT generated by reading the image
% file names but by using start, end times and plot distance - the same
% method the plotting script does.
% The start and stop times here should be identical to the ones used to
% generate the plots
% The script will stop when it generates a name tha does not match any of
% the image files. Adjust the start time to the next existing figure and
% run agian to generate another txt file.

clc; close all; clearvars;
plotterVersion = 2.1;
%% Data path and plot times
% Update the path and plot times here to plot the entire directory without a promt with QuickPlotAll = true

Campaign = 'AN02';  % This is the filekey
dataPath = 'E:\AN1\theater\Antarctica\20152016\level1\SIR\Stack04\F1007\mat\';
outputpath = 'E:\AN1\theater\Antarctica\20152016\level1\SIR\Stack04\F1007\';     % comment this to save figures in dataPath\plots\

startplottime = '03-Dec-2015 23:20:15';
endplottime = '07-Dec-2015 08:14:01';

%% Plot Configurations
QuickPlotAll = true;        % enable this to plot all files in the directory without prompt
multiplePlots = true;       % setting this to 0 will generate single plot from start time to end time

% plotSig = {'3usCh1'};       % one of '3usCh1', '3usCh2', '1usCh1' or '1usCh2 Keep this format - no spaces

surfZero = true;      % Sets zero at surface at the begining of the plot
refsurf = 2500;        % ft, below the aircraft- if no surface is found this will be used

% axes and tick labels
plotXaxis = 'dist';         % 'dist' or 'time'. Will plot data against distance or time. Both will be displayed, if available.
displayAxes = {'time','pos','dist'}; % 'time','dist','pos' ->time in HH:MM, distance in km, Lat/Lon in degrees
plotPeriod = 60;            % minutes or km depending on plotXaxis type. Only used if multiplePlots = true;
showScale = false;           % This will put the distance scale in the image

% surfaces and layers
surfType = 'none';          % 'first','max','both' or 'none'. first = closest to aircraft, max = maximum power
surfLineColor = {'m-','g-'};            % LineSpec for surface(s)
surfLegend = {'First','Max Power'};     % Legends for surface line(s)
legendPos = 'NorthEast';   % legend location - try help legend

% Plot power level
plotPower = false;
powerSmoothWin = 256;
% figure save parameters
contUpdateFig = false;  % draws the figure every file - slower
printFigure = true;      % clearing this will plot wihout saving the pngs
pngpagesize = [15,8];      % inches
pdfpagesize = [11,8];      % inches
pngresolution = 600;       % dpi
pdfresolution = 300;       % dpi
% renderer for bitmap for pdf
if verLessThan('matlab','8.4') % earlier than 2014b
    pdfrenderer = 'zbuffer';
else
    pdfrenderer = 'opengl';
end
pngrenderer = 'painters';
outputpdf = true;       % create pdf of image
outputpng = true;       % create pnf of image

% Pop Layers using vertical derivative
x_smooth_win = 1;
y_smooth_win = 1;

diffPlot = false;
diffOrder = 1;

detrendy = false;
fftAvg = 1;
rangeAvg = 1;

closeFigures = false;   % closes figures to prevent filling up the memory.

%% Additional Info
surveySpeed = 100;          % m/s, will be used as backup if no gps is available.

%% This is for plotting already-picked layers in the echogram
% plot other available surfaces - these are lines across the image
% each line is required to have a time vector and a surface vector
% the variable names, data files and location is defined here
% The data is assumed to be a single mat file for the entire flight/line
% plotted. Time should be in unix time and layer data in index number
% of the radar echo

plotLayers = false;      % flag to enable plotting of layers
numLayers = 1;           % number of additional surface to plot

% For each additional surface assign the following by incrementing the
% cell index

%%%%%%%%%% Layer 1 %%%%%%%%%%%%%%%%%%

layerDataPath{1} = 'E:\DiceTestDataRS1\theater\Antarctica\20152016\level1\DICE\Stack64\F1004\layer1.mat';   % full path and name of the layer mat file
layerVar{1} = 'layerInd';           % variable name for the surface
layerTimeVar{1} = 'echotimeutc';    % time variable for the layer - this need to be in utc seconds
layerLegend{1} = 'Layer1';          % label for the layer
layerLineSpec{1} = 'r*';            % Line specification for the layer to be plotted
layerDataType{1} = 'RangeDist';    	% 'Index','RangeDist','DepthDist','RangeTime','DepthTime','Elev'
%%%%%%%%%% Layer n %%%%%%%%%%%%%%%%%%

% Repeat info above for additional layers
% layerDataPath{n} = '';   % full path to the surface file
% and so on..

%% This whole section needs to be cleaned up --- later
pdfoutputpath = [outputpath filesep 'pdf' filesep];
pngoutputpath = [outputpath filesep 'png' filesep];
txtoutputpath = [outputpath filesep 'txt' filesep];
pdfFlist = [];
pngFlist = [];
if exist(pdfoutputpath,'dir')==7
    pdfFlist = dir([pdfoutputpath,'*.pdf']);
end
if exist(pngoutputpath,'dir')==7
    pngFlist = dir([pngoutputpath, '*png']);
end
if isempty(pdfFlist) && isempty(pngFlist)
    error('No pdf or png files found. Please check the output directory');
elseif numel(pngFlist) ~= numel(pdfFlist)
    warning('sirGenFig:misMatchedFigs','Number of pdf (%d) and png (%d) files dont match',numel(pdfFlist),numel(pngFlist))
    if numel(pngFlist)>numel(pdfFlist)
        for iif = 1:numel(pngFlist)
            [~,imageFlist{iif},~]= fileparts(pngFlist(iif).name);
        end
    else
        for iif = 1:numel(pdfFlist)
            [~,imageFlist{iif},~]= fileparts(pdfFlist(iif).name);
        end
    end
else
    for iif = 1:numel(pngFlist)
        [~,imageFlist{iif},~]= fileparts(pngFlist(iif).name);
    end
end
%% Figure stuff
clims = [-20,60];       % Change limits to change image contrast
ylims = [300, -600];  	% [above_surface below_surface] meters in ice about the surface
cmap = 'gray';          % valid colormap string
invertcaxis = true;    	% 0 or 1
reversexaxis = false; 	% reverse x axis direction
cutterH = 1;            % horizontal decimation
if diffPlot             % set color limits for diff plot
    clims = [-1 1];
end
%% Radar parameters and constants
cAir = 299792458;                     % m/s
cIce = 1.68e8;                        % m/s
%%  Don't touch these
outFileDateFormat = 'yyyymmdd';
outFileTimeFormat = 'HHMMSS';
dateField = 3;
timeField = 4;
fileFieldSep = '_';

if exist('userRefElev','var')
    userRefElev = userRefElev/3.28;
end
refsurf = refsurf/3.28*cIce/cAir;
layerData = cell(1,numLayers);
layerTime = cell(1,numLayers);
xaxisDataLayer = cell(1,numLayers);
for nnl = 1:numLayers
    layerData{nnl} = NaN;
    layerTime{nnl} = NaN;
    xaxisDataLayer{nnl} = NaN;
end
layersLoaded = false;
allechotimeutc = [];
allhell = [];
alllat = [];
alllon = [];
allSurfIndRad = [];
allSirPwrOut = [];
maxXticksNum =9;
clearvars nnl;
%%
if ~printFigure
    warning('sirPlotTool:figureNotSaved','Figures will not be saved! \nset savefigure = true to save figures')
end

%%
if (QuickPlotAll)
    if ~(dataPath(end) == filesep)
        dataPath = [dataPath filesep];
    end
    if ~(exist(dataPath,'dir')==7)
        warning('sirPlotTool:noValidDataPath','Data path %s is not valid',dataPath);
        QuickPlotAll = false;
    end
end

if  ~QuickPlotAll
    dataPath = uigetdir(pwd,'Select dice processed data directory to plot');
    if isempty(dataPath) || numel(dataPath)<2
        error('sirPlotTool:noDirSelected','Aborted: no directory selected');
    end
    dataPath = [dataPath filesep];
end
tic;
fprintf('Plotting files in dir: %s\n',dataPath);

% Find and sort data files
level1Files = dir([dataPath,['*',Campaign,'*.mat']]);
numfiles=numel(level1Files);
if(numfiles == 0)
    error('sirPlotTool:noDataToPlot','No data available to plot\n Please check the data dir');
else
    for ii = 1:numfiles
        fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
        level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
    end
    [~,sx] = sort([level1Files.utc]);
    level1Files = level1Files(sx);
end

ftimeutc = [level1Files.utc];
numfiles=numel(level1Files);
if numel(fileNameFields)>5
    lastFileField = strsplit(fileNameFields{end},'.');
    lastFileField = [fileNameFields{end-1},'_',lastFileField{1}];
else
    lastFileField = [];
end

if numfiles>1
    filePeriod = mode(round(diff([level1Files.utc])));
else
    load([dataPath,level1Files.name],'echotimeutc');
    filePeriod = range(echotimeutc); clearvars echotimeutc;
end

% Allow user to select start and end plot times
if(~QuickPlotAll)
    prompt = {'Plot start time:','Plot end time:'};
    dlg_title = 'Plot Range';
    firstfiletime = unix2matlabtime(ftimeutc(1));
    lastfiletime = unix2matlabtime(ftimeutc(numfiles) + filePeriod);
    def = {datestr(firstfiletime),datestr(lastfiletime)};
    answer = inputdlg(prompt,dlg_title,1,def);
    if isempty(answer)
        error('Aborted: no range selected')
    else
        startplottime = answer{1};
        endplottime = answer{2};
    end
end

startplotutc=matlab2unixtime(datenum(startplottime));
endplotutc=matlab2unixtime(datenum(endplottime));
%% Select files to plot
startplotfile=find(ftimeutc <= startplotutc,1,'last');
endplotfile=find(ftimeutc > endplotutc,1,'first');

if isempty(startplotfile)
    startplotfile=1;
    startplotutc = level1Files(1).utc;
end
if isempty(endplotfile)
    endplotfile=numfiles;
    endplotutc = ftimeutc(numfiles) + filePeriod;
else
    endplotfile = endplotfile-1;
end

if (endplotfile - startplotfile)< 1
    warning('Start and End times are not correct')
    prompt = {'Plot start time:','Plot end time:'};
    dlg_title = 'Plot Range';
    firstfiletime = unix2matlabtime(ftimeutc(1));
    lastfiletime = unix2matlabtime(ftimeutc(numfiles) + filePeriod);
    def = {datestr(firstfiletime),datestr(lastfiletime)};
    answer = inputdlg(prompt,dlg_title,1,def);
    if isempty(answer)
        error('Aborted: so range selected')
    else
        startplottime = answer{1};
        endplottime = answer{2};
    end
    startplotutc=matlab2unixtime(datenum(startplottime));
    endplotutc=matlab2unixtime(datenum(endplottime));
    
    startplotfile=find(ftimeutc <= startplotutc,1,'last');
    endplotfile=find(ftimeutc <= endplotutc,1,'last');
    
    if isempty(startplotfile)
        startplotfile=1;
        startplotutc = ftimeutc(1);
    end
    if isempty(endplotfile)
        endplotfile = numfiles;
        endplotutc = ftimeutc(end) + filePeriod;
    end
end

if endplotfile - startplotfile + 1 < numel(ftimeutc)
    warning('sirPlotTool:notAllFilesPlotted',...
        'Not all available data are plotted. Change start/stop times to plot all available data');
end

if ismember('dist',displayAxes) || ismember('pos',displayAxes) || strcmpi(plotXaxis,'dist')
    distlat = [];
    distlon = [];
    disthell = [];
    distechotimeutc = [];
    for dff = startplotfile:endplotfile
        load([dataPath,level1Files(dff).name],'echotimeutc','lat','lon','hell');
        if exist('echotimeutc','var') && exist('lat','var') && exist('lon','var') && exist('hell','var')
            distlat = [distlat lat]; clearvars lat;
            distlon = [distlon lon]; clearvars lon;
            disthell = [disthell hell]; clearvars hell;
            distechotimeutc = [distechotimeutc echotimeutc]; clearvars echotimeutc;
        else
            warning('sirPlotTool:noGPSData','no GPS data available for one or more files');
            warning('off','sirPlotTool:noGPSData');
        end
    end
    inrangeInd = distechotimeutc >= startplotutc & distechotimeutc <= endplotutc;
    distechotimeutc = distechotimeutc(inrangeInd);
    distlat = distlat(inrangeInd);
    distlon = distlon(inrangeInd);
    disthell = disthell(inrangeInd);
    
    % if any Nan found in position data
    if any(isnan(distlat)) || any(isnan(distlon)) || any(isnan(disthell))
        maxNumNanVals = max([numel(find(isnan(distlat))),numel(find(isnan(distlat))),numel(find(isnan(distlat)))]);
        %if more than 20% of position data are Nan revert to plotting by
        %time
        if maxNumNanVals>0.2*numel(distechotimeutc)
            warning('SirPlotTool:tooManyNanPosData','Number of NaN position data exceeds 20%%, plot x-axis will be switched to time')
            plotXaxis = 'time';
        else
            % find the first and last data points those are not NaN
            firstNotNan = find(~isnan(distlat + distlon + disthell),1,'first');
            lastNotNan = find(~isnan(distlat + distlon + disthell),1,'last');
            startplotutc = distechotimeutc(firstNotNan);
            endplotutc = distechotimeutc(lastNotNan);
            distlat = distlat(firstNotNan:lastNotNan);
            distlon = distlon(firstNotNan:lastNotNan);
            disthell = disthell(firstNotNan:lastNotNan);
            distechotimeutc = distechotimeutc(firstNotNan:lastNotNan);
            % find files within the time range
            startplotfile=find(ftimeutc <= startplotutc,1,'last');
            endplotfile=find(ftimeutc <= endplotutc,1,'last');
            % interpolate out any Nan values within within the position
            % values
            if any(isnan(distlat))
                distlat(isnan(distlat)) = interp1(echotimeutc(~isnan(distlat),distlat(~isnan(distlat)),echotimeutc(isnan(distlat))));
            end
            if any(isnan(distlon))
                distlon(isnan(distlon)) = interpDegWithJumps(echotimeutc(~isnan(distlon),distlon(~isnan(distlon)),echotimeutc(isnan(distlon))));
            end
            if any(isnan(disthell))
                distlon(isnan(disthell)) = interp1(echotimeutc(~isnan(disthell),disthell(~isnan(disthell)),echotimeutc(isnan(disthell))));
            end
        end
    end
    
    if isempty(distlat) || isempty(distlon) || isempty(disthell) || isempty(distechotimeutc)
        displayAxes = {'none'};
        if strcmpi('dist',plotXaxis)
            warning('sirPlotTool:noGPSDataForAllFiles','no GPS data available for any file!\n plot x-axis will be changed to time')
            plotXaxis = 'time';
            plotPeriod = round(plotPeriod*1000/surveySpeed/60); % km*1000/m/s/60  km -> min
        end
    else
        %%%%% This bit implements a fix for jumps in longitude
        distlon(distlon<0) = distlon(distlon<0)+360;
        dLonInd = find(abs(diff(distlon))>1);    % Any jump more than 1 deg
        if isempty(dLonInd)
            distlon(distlon>180) = distlon(distlon>180)-360;
        else
            dTimeInd = find(diff(distechotimeutc)>filePeriod/5);      % Any jump more than ~ 2s
            if ~isempty(dTimeInd)
                warning('sirPlotTool:dataGapsFound','Gaps found in data - review flight logs')
                for dgg = 1:numel(dTimeInd)
                    fprintf('Data gaps found at: %s\n',datestr(unix2matlabtime(distechotimeutc(dTimeInd(dgg)))));
                end
            end
            dLonInd = dLonInd(~ismember(dLonInd,dTimeInd));     % find all the longitude indices that are not same as time indices
            if isempty(dLonInd)
                distlon(distlon>180) = distlon(distlon>180)-360;
            else
                warning('sirPlotTool:lonJumpFound',['%d jumps found in longitude for %s \n',...
                    'Re-run diceGeoLocate for level1 data\n',...
                    'Re-process SAR with neighboring files and replace the file in concern\n',...
                    'Plotting will continue..'],numel(dLonInd),fileNameFields{2});
                fprintf('Longitude jump found at:\n');
                for ljj = 1: numel(dLonInd)
                    fprintf('%s\n',datestr(unix2matlabtime(distechotimeutc(dLonInd(ljj)))));
                end
                dLonInd = unique([dLonInd(dLonInd ==1); dLonInd(dLonInd(1:end-1)+1 == dLonInd(2:end));dLonInd(dLonInd ==numel(distlon)-1)]);
                midInds = dLonInd(dLonInd > 1 & dLonInd<numel(distlon)-1);
                distlon(midInds+1) = 0.5*(distlon(midInds) + distlon(midInds+2));
                % Take care of the case when the jump is the first or last data point
                if any(dLonInd ==1)
                    distlon(1) = interpDegWithJumps(distechotimeutc(2:3),distlon(2:3),distechotimeutc(1),'linear','extrap');
                end
                if any(dLonInd ==dLonInd<numel(distlon))
                    distlon(end) = interpDegWithJumps(distechotimeutc(end-2:end-1),distlon(end-2:end-1),distechotimeutc(end),'linear','extrap');
                end
                distlon(distlon>180) = distlon(distlon>180)-360;
            end
        end
        %%%%%%%%
        [dx,dy,dz] = geodetic2enu(distlat(2:end),distlon(2:end),disthell(2:end),...
            distlat(1:end-1),distlon(1:end-1),disthell(1:end-1),wgs84Ellipsoid,'degrees');
        dHorzDist = ([0 dx].^2 + [0 dy].^2).^0.5; % first dx,dy,dz
        distall = cumsum(dHorzDist)/1000;
    end
end

if(strcmpi(plotXaxis,'dist'))
    plotbydist = true;
else
    plotbydist = false;
end

% set number of x ticks base on x-axis data and plot period
if plotbydist
    numXticks = min(max(min(ceil(plotPeriod/10)+1,10),5),maxXticksNum);   % number of x-axis ticks
else
    numXticks = min(max(min(ceil(plotPeriod/5)+1,5),5),maxXticksNum);   % number of x-axis ticks
end


if multiplePlots
    if plotbydist
        numbatches = ceil(distall(end)/plotPeriod);
        if numbatches > (endplotfile-startplotfile-1)
            plotPeriod = round(filePeriod*2*surveySpeed/1000);
            warning('sirPlotTool:batchTooShort',['plotPeriod should be larger than a file size.\n',...
                'Changing plot period to %d'],plotPeriod);
        end
        batchstartutc(1) = startplotutc;
        if(numbatches == 1)
            batchendutc(1) = endplotutc;
        else
            batchMaxInd = find(distall<= plotPeriod,1,'last');
            batchendutc(1) = distechotimeutc(batchMaxInd);
            for bb = 2:numbatches - 1
                batchstartutc(bb) = distechotimeutc(batchMaxInd+1);
                batchMaxInd = find(distall <= bb*plotPeriod,1,'last');
                batchendutc(bb) = distechotimeutc(batchMaxInd);
            end
            batchstartutc(numbatches) = distechotimeutc(batchMaxInd+1);
            batchendutc(numbatches) = endplotutc;
        end
    else
        batchstartutc = startplotutc:plotPeriod*60:endplotutc;
        batchendutc = min([batchstartutc+plotPeriod*60-1],endplotutc);
        numbatches = numel(batchstartutc);
    end
else
    batchstartutc = startplotutc;
    batchendutc = endplotutc;
    numbatches = 1;
    numXticks = maxXticksNum;
end

for bb = 1:numbatches
    if bb ==1
        batchmin(bb) = startplotfile;
    else
        batchmin(bb) = min(batchmax(bb-1) + 1,endplotfile);
    end
    if bb == numbatches
        batchmax(bb) =  endplotfile;
    else
        batchmax(bb) =  find(ftimeutc <= batchendutc(bb),1,'last');
    end
    if batchmax(bb) == endplotfile && bb < numbatches
        numbatches = bb;
        break;
    end
end

minfNum=batchmin;
maxfNum=batchmax;

numExcessFrames = 0;
figHandle = -ones(numbatches,1);
for batch=1:numbatches
    fprintf('Processing figure name %d of %d\n',batch,numbatches)
    for fnum = minfNum(batch):maxfNum(batch)
        load([dataPath,level1Files(fnum).name],...
            'echotimeutc','hell','lat','lon');
        if fftAvg >1
            numframes = numel(echotimeutc) - mod(numel(echotimeutc),fftAvg);
            echotimeutc = mean(reshape(echotimeutc(1:numframes),fftAvg,numframes/fftAvg),1);
            lat = mean(reshape(lat(1:numframes),fftAvg,numframes/fftAvg),1);
            lon = mean(reshape(lon(1:numframes),fftAvg,numframes/fftAvg),1);
            hell = mean(reshape(hell(1:numframes),fftAvg,numframes/fftAvg),1);
        end
        % Compatibility with older data
        if exist('hell','var') && exist('lat','var') && exist('lon','var')
            gpsexist = true;
        else
            warning('off','MATLAB:load:variableNotFound');
            gpsexist = false;
        end
        if  fnum == minfNum(batch)  % First file of a batch
            % clear last surface and time from previous files- These are
            % plotted with current surfaces to eliminate gaps in the
            % surface plots
            %             lastSurfRange = [];
            %             lastSurfTime = [];
            if batch > 1 && numExcessFrames > 0 % Add leftover echoes from last batch
                echotimeutc = [excessechotimeutc(1:numExcessFrames) echotimeutc];
                if gpsexist
                    lat = [excesslat(1:numExcessFrames) lat];
                    lon = [excesslon(1:numExcessFrames) lon];
                    hell = [excesshell(1:numExcessFrames) hell];
                end
                if ~isempty(echotimeutc)
                    batchfirstutc = echotimeutc(1);
                else
                    break;
                end
            else %Very first file, find echoes in plot range
                inTimeEchoes = echotimeutc>=batchstartutc(batch);
                echotimeutc = echotimeutc(inTimeEchoes);
                if gpsexist
                    lat = lat(inTimeEchoes);
                    lon = lon(inTimeEchoes);
                    hell = hell(inTimeEchoes);
                end
                if ~isempty(echotimeutc)
                    batchfirstutc = echotimeutc(1);
                else
                    warning('First time not set. Updating batch list')
                    minfNum(batch) = fnum+1;
                    break;
                end
            end
            if  gpsexist
                reflat = lat(1);
                reflon = lon(1);
                reftime = echotimeutc(1);
                if ~exist('userRefElev','var')
                    refelev = hell(1);
                else
                    refelev = userRefElev;
                end
            end
            
        end
        
        if fnum == maxfNum(batch)
            inTimeEchoes = echotimeutc<=batchendutc(batch);
            if batch < numbatches   % last file of a batch, store any excess frames for next batch
                excessechotimeutc = echotimeutc(~inTimeEchoes);
                if gpsexist
                    excesslat = lat(~inTimeEchoes);
                    excesslon = lon(~inTimeEchoes);
                    excesshell = hell(~inTimeEchoes);
                else
                    excesslat = ones(size(excessechotimeutc))*NaN;
                    excesslon = ones(size(excessechotimeutc))*NaN;
                    excesshell = ones(size(excessechotimeutc))*NaN;
                end
                numExcessFrames = numel(find(~inTimeEchoes));
            else % very last batch ,
                numExcessFrames = 0;
            end
            echotimeutc = echotimeutc(inTimeEchoes);
            if gpsexist
                lat = lat(inTimeEchoes);
                lon = lon(inTimeEchoes);
                hell = hell(inTimeEchoes);
            end
            if isempty(echotimeutc)
                break;
            end
        end
        if plotbydist
            allechotimeutc = [allechotimeutc echotimeutc];   % accumulate for layer plotting
            allhell = [allhell hell];
            alllat = [alllat lat];
            alllon = [alllon lon];
        end
    end
    allSurfIndRad = [];
    if ~isempty(fnum)
        Campaign = fileNameFields{1};
        FlightNo = fileNameFields{2};
        
        distbatch = interp1(distechotimeutc,distall,allechotimeutc);
        distbatch = distbatch - distbatch(1);
        batchhell = allhell([1,end]);
        batchechotime = allechotimeutc([1,end]);
        batchlat = alllat([1,end]);
        batchlon = alllon([1,end]);
        
        allhell = [];
        allechotimeutc = [];
        alllat = [];
        alllon = [];
        allSirPwrOut = [];
        
    end
    if ~exist('datatag','var')
        datatag = lastFileField;
    end
    
    pngfigname = [pngoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,datatag,'.png'];
    
    
    pdffigname = [pdfoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,datatag,'.pdf'];
    
    % Save figure:
    if exist('pngfigname','var')
        [~,figname,~] = fileparts(pngfigname);
    else
        [~,figname,~] = fileparts(pdffigname);
    end
    
    try
        figInfo(batch,:) = {figname,datestr(unix2matlabtime(batchechotime(1)),'mm/dd/yyyy,HH:MM:SS'),batchechotime(1),...
            datestr(unix2matlabtime(batchechotime(2)),'mm/dd/yyyy,HH:MM:SS'),batchechotime(2),...
            batchlat(1),batchlon(1),batchhell(1),batchlat(2),batchlon(2),batchhell(2)};
    catch
        warning('sirPlotTool:noPosInfo','Position info is not Available');
        warning('off','sirPlotTool:noPosInfo');
        figInfo(batch,:) = {NaN,NaN,NaN,NaN,NaN,NaN,NaN,NaN,NaN};
    end
    
    if exist(txtoutputpath,'dir')==0
        mkdir(txtoutputpath);
    end
    txtfilename = [txtoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(startplotutc),...
        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,datatag,'.txt'];
    if ~exist('txtfid','var')
        txtfid = fopen(txtfilename,'w');
        % cleans the fileID when the function exits.
        cleaner = onCleanup(@()fclose(txtfid));
        fprintf(txtfid,'FigureName, StartUTCDate,StartUTCTIme,StartUnixTime,StopUTCDate,StopUTCTIme,StopUnixTime,StartLatitude,StartLongitude,StartH-El,StopLatitude,StopLongitude,StopH-El\n');
    end
    if ismember(figname,imageFlist)
        fprintf(txtfid,'%s,%s,%.0f,%s,%.0f,%0.6f,%0.6f,%0.3f,%0.6f,%0.6f,%0.3f\n',figInfo{batch,:});
    else
        clearvars cleaner;
        if batch>1
            error('sirGenFigTxt:fignameMismatch',['Figure name does not match. Writing will be stopped\n',...
                'Last matching figure name written is %s. Adjust the start time and run the script again for remaining Figures'],figInfo{batch-1,1})
        else
            error('sirGenFigTxt:noMatchingFigNames','First figure name %s does not match with existing files\nAdjust the start time to match the first image file',figInfo{batch,1})
        end
    end
end
clearvars cleaner;
warning('on','all')
toc