% Tejendra Dhakal
% 02/18/2014
% LDEO, Columbia University
%09/08/2015 Added first pass motion compensation on dechirped data

% Parse, FFT and plot SIR raw files
% Dependencies:
% Fb2Range.m - Function that computes range from frequncy,chirpperiod etc
%%
clc; clearvars; close all;
tic
%% Processing Flags
gpu_process = 0;
if(gpu_process)
    reset(gpuDevice(1))
end
SaveData = 1;
plotFigure = 0;
SaveFigure = 0;
fignum = 1;
contFigUpdate = false;
smooth_plot = 1;
smoothwin_x = 8;       % default 8
smoothwin_y = 2;        % default 2
normalizeEchoes = false;    %normalizes echoes by its average power
diffplot = 0;
difforder = 1;
rangeLim = [400 550];
% cLims = [-70 -20];
motionComp = 1;                  	% Uses GPS data to compensate for aircraft bulk motion deltaR
rvp_corr = 1;
lowMem = 1;                       	% Uses for loop for fft
remove_meansig = 1;                	% Removes mean signal over 2 km (first 2 files in the folder) before fft
numCohAvg = 16;                     % numbers of chirps to average before fft
numfftAvg = 2;                  	% numbers of chirps to average after fft
numAvg = numCohAvg*numfftAvg;
fftwintype = 'kaiser';            	% 'cheb','kaiser','hanning'
fftwinparam = 3;                  %  100 for cheb, 5-15 for kaiser
starttime='12-01-2014 09:38:00';    % UTC time of data to start processing at.  Format: 'dd-mm-yyyy HH:MM:SS'
endtime='12-01-2014 09:40:00';      % UTC time of data to end processing at.  Format: 'dd-mm-yyyy HH:MM:SS'

usefilter = 1;
filtertype = 'low';        % 'low' or 'bandpass' or 'none'
RangePass = [400 550];              %m by Range
filtOrder = 8;

findLayers = false;                  % finds peaks and sorts them by power
numLayers = 10;                      % finds 
layerWin = [400 550];               % Range window to find layer in
plotLayers = [];                 % layers to plot
layerSmoothWin = smoothwin_x;
layerLineSpec = {'y','m','g','c','b'};
layerName = {'Layer1','Layer2','Layer3','Layer4','Layer5'};
%% diagnostic
checkDataAlign = false;
checkHeader = false;
%%%%%%% Flight Parameters%%%%%%%%%%%%%%%%%%%%%%%
Campaign = 'AN01';                  % Name of Campaign
FlightNo = 'F1006';
FlightDesc = 'TNB';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File Parameters
inputfolder = 'E:\AN1\theater\Antarctica\20142015\raw\SIR\F1006\';
outputfolder = 'E:\AN1\theater\Antarctica\20142015\raw\SIR\F1006\Processed16-2\';
gpsfile= 'E:\AN1\theater\Antarctica\20142015\level1\GNSS-IMU\F1006\mat\AN1-F006-20141201-SPAN_FINAL.mat';  % required if motionComp = 1
if(ispc)
    if ~exist('inputfolder','var')
        inputfolder = uigetdir('Select the SIR data Dir to Process');
    else if ~exist(inputfolder,'dir')
            inputfolder = uigetdir('Select the SIR data Dir to Process');
        end
    end
    inputfolder = [inputfolder filesep];
    if ~exist('outputfolder','var')
        outputfolder = uigetdir('Select the Dir for processed Data');
        outputfolder = [outputfolder filesep];
    end
    if (motionComp == 1)
        if ~exist('gpsfile','var')
            [gpsfile, gpsfilepath]= uigetdir('Select the GPS file for this flight','*.mat');
            gpsfile = fullfile(gpsfilepath, gpsfile);
        else  if ~exist(gpsfile,'file')
                [gpsfile, gpsfilepath]= uigetfile('*.mat','Select the GPS file for this flight');
                gpsfile = fullfile(gpsfilepath, gpsfile);
                clear gpsfilepath
            end
        end
    end
end
%Load GPS data
load(gpsfile,'dateUTC','sodUTC','Lat','Lon','Alt','Heading','Pitch','Roll');
GPSUTC = 86400*((dateUTC + sodUTC/86400) - datenum('01-01-1970'));
titletext = {['SIR ' FlightNo ' ' FlightDesc];
    [num2str(numCohAvg,'-CohAvg=%d, ') num2str(numfftAvg,'-fftAvg=%d-')]};
if smooth_plot
    titletext = [titletext;[num2str(smoothwin_x,'xwin =%i,'),num2str(smoothwin_y,'ywin =%i,')]];
end

filenamedateinds = [11,18];           % Indices of datestamp within input file names (two integers)
filenametimeinds = [20,25];          % Indices of timestamp within input file names (two integers)
file_period = 10;                    % file length in seconds
%%%%%%%%%%%%%%%%%%%% Radar Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fs = 12.5e6;        % Sampling Frequency
tau = 750e-6;        % Chirp Period
bw = 599996441;     % Chirp bandwidth Hz
% fs = 50e6;        % Sampling Frequency
% tau = 250e-6;        % Chirp Period
% bw = 600008082;     % Chirp bandwidth Hz
Kr = bw/tau;
f0 = 1.7e9;
tsamp = linspace(0,tau-1/fs,fs*tau)';
fft_period = 1*tau;  % chirp period is
framelen = 9352;    % number of samples collected for each chirp
sigRange = 500:8800;  % discard first and last 2 samples (Marker Bytes)
tsamp = tsamp(sigRange);
fftdatalen = numel(sigRange);
conversion_factor = 2.415/2^16; % adc conversion into volts
%%%%%%%%%%%%%%%%%%%% Constants %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c_ice = 1.68e8;     % speed of light in ice m/s
c = 299792458;      % speed of light in vacuum m/s
%%
dFileStr = dir([inputfolder,'*.sir']);
numDataFiles = length(dFileStr);
if numDataFiles==0
    error('Data Files Not Found!')
end
% Sort data files by timestamp:
for nfiles = 1:numDataFiles
    dFileStr(nfiles).utc=86400*...
        (datenum([dFileStr(nfiles).name(filenamedateinds(1):filenamedateinds(2)),...
        dFileStr(nfiles).name(filenametimeinds(1):filenametimeinds(2))],'yyyymmddHHMMSS')...
        -datenum('01-01-1970 00:00:00','dd-mm-yyyy HH:MM:SS'));
end
[sv,sx] = sort([dFileStr.utc]);
dFileStr = dFileStr(sx);
fTime = [dFileStr.utc];

% Find any file with missing frames
truncatedFile = [dFileStr.bytes] < framelen*numAvg;
missingFile = [false diff([dFileStr.utc])>11];      % This determines if files are contiguous

% Figure out which input files to process:
startutc=matlab2unixtime(datenum(starttime));
startfile=find([dFileStr.utc]<=startutc,1,'last');
endutc= matlab2unixtime(datenum(endtime));
endfile=find([dFileStr.utc]>=endutc,1,'first');

if isempty(startfile)
    startfile=1;
end
if isempty(endfile)
    endfile=numDataFiles;
end
startfile = 1;
endfile = numDataFiles;
% if(remove_meansig == 1)
%     if(~exist([inputfolder 'meanSirData.mat'],'file'))
%         fprintf('Mean signal not found - Creating...\n');
%         makeMeanSig(inputfolder,dFileStr,framelen,startfile,2);
%     end
%     if(lowMem ==1)
%         load([inputfolder 'meanSirData.mat'],'meanSirSig');
%         meanSirSig = hilbert(meanSirSig(sigRange-2)*conversion_factor);
%     else
%         load([inputfolder 'meanSirData.mat'],'meanSirMat');
%         meanSirMat = hilbert(meanSirMat(sigRange-2,:)*conversion_factor);
%     end
% end
% Make output folder, if necessary:
if exist(outputfolder,'dir')==0
    mkdir(outputfolder)
end
%% Compute Range and Frequency bins
NFFTr = max(2^nextpow2(fftdatalen),32768);
fbins = fs/2*linspace(0,1,NFFTr/2+1)';
fbins_rvp = fs/2*linspace(-1,1,fftdatalen)';  % this carries positive part of the spectrum including 0
twowaytime = fbins*tau/bw;
% InRange = 1:NFFTr/2;
% fbins = fbins(InRange);
RangeFull = Fb2Range(bw,tau,fbins);
InRange = find(RangeFull>= rangeLim(1) & RangeFull <= rangeLim(2));
fbins = fbins(InRange);
Range = RangeFull(InRange);
fpass = fbins([find(Range>=RangePass(1),1,'first'),find(Range<=RangePass(2),1,'last')]);
if usefilter
    if strcmpi(filtertype,'low')
        [z,p,k] = butter(filtOrder,fpass(2)/(fs/2),'low');
    else
        [z,p,k] = butter(filtOrder,fpass/(fs/2),'bandpass');
    end
    sirFilt = zp2sos(z,p,k);
end

maxLinesperFile = ceil(max([dFileStr.bytes]/2/framelen))+ numAvg;
% Pre-allocate
if(gpu_process == 1)
    tempFFTdata = gpuArray(zeros(fftdatalen,ceil(maxLinesperFile/numCohAvg)));
end
tempFileData = uint16(zeros(maxLinesperFile*framelen,1));
excessData = uint16(zeros(framelen,numAvg));
tempData = zeros(fftdatalen,maxLinesperFile);
excessframes = 0;
% This creates logical index for entire file to extract data in one go
dataind = repmat(logical([zeros(2,1);zeros(sigRange(1)-3,1);ones(fftdatalen,1);zeros(framelen-sigRange(end),1)]),maxLinesperFile,1);
if checkHeader
    headerind = repmat(logical([ones(2,1);zeros(framelen-2,1)]),maxLinesperFile,1);
end
sig = zeros(fftdatalen,maxLinesperFile);
sif2 = zeros(NFFTr,maxLinesperFile);
% sirfftout = zeros(length(InRange),ceil(maxLinesperFile/numAvg));
sirPwrEcho = zeros(1,ceil(maxLinesperFile/numAvg));
if ~lowMem
    H_MC = zeros(fftdatalen,ceil(maxLinesperFile));
    del_tau = zeros(fftdatalen,ceil(maxLinesperFile));
else
    del_tau = zeros(1,ceil(maxLinesperFile));
end

fftwintype = lower(fftwintype);
switch fftwintype
    case 'cheb'
        fftwin = chebwin(fftdatalen,fftwinparam);
    case 'kaiser'
        fftwin = kaiser(fftdatalen,fftwinparam);
    case 'hanning'
        fftwin = hann(fftdatalen);
    case 'none'
        fftwin = 1;
    otherwise
        fftwin = hann(fftdatalen);
        warning('sirParserFFTandPlot:invalidWin',...
            'Window type %s not recognized - a hanning window will be used',fftwintype);
end

if ~lowMem
    fftwinMat = repmat(fftwin,1,maxLinesperFile);
    tsampMat_full = repmat(tsamp,1,ceil(maxLinesperFile));
end

if(rvp_corr ==1)
    if(lowMem == 1)
        rvpCorr = exp(-1j*pi*fbins_rvp.^2/(bw/tau));
    else
        rvpCorrMat = repmat(exp(-1j*pi*fbins_rvp.^2/(bw/tau)),1,maxLinesperFile);
    end
end

figHandle = [];
isfirstfile = 1;
lastfileutc = dFileStr(startfile).utc;
lastechotime = [];
fnum = startfile;
fixThisFile = 0;
dotdiv = @(a,b) a./b;
dotmult = @(a,b) a.*b;
while  fnum <= endfile
    if(dFileStr(fnum).bytes/2 <framelen)
        warning('Skipping Truncated File %s\n Size smaller than fft_period: %d',...
            dFileStr(fnum).name,dFileStr(fnum).bytes/2);
        fnum = fnum +1;
        % discard the echoes from previous files and clear time of last
        % echo
        excessframes = 0;
        lastechotime = [];
        continue
    end
    % check if last file is older than 10 seconds.. if it is get rid of
    % the excess data
    if fnum>startfile && excessframes >0
        if((dFileStr(fnum).utc - lastechotime) <0.25); % if missing more than 0.25 of frames reset the time 
            tempFileData(1:excessframes*framelen) = excessData(1:excessframes*framelen);
        end
    else
        % discard the echoes from previous files and clear time of last
        % echo
        excessframes = 0;
        lastechotime = [];
    end
    
    numframesInFile = dFileStr(fnum).bytes/2/framelen;
    if(mod(numframesInFile,1)>0 || fixThisFile)
        numframesInFile = floor(numframesInFile);
        warning('Partial Frames Found in File %s\n',dFileStr(fnum).name);
        fprintf('Recovering Frames...\n');
        dFileStr(fnum).bytes = fixSirFile([inputfolder,dFileStr(fnum).name],framelen);
        if fixThisFile
            fixThisFile = 0;
        end
        continue
    end
    framesToRead = (numframesInFile) - mod(excessframes + numframesInFile,numAvg);
    fprintf('Processing File %d of %d\n',fnum-startfile+1,endfile-startfile+1);
    numframes = excessframes + framesToRead;
    
    % compute time for each echo
    if isempty(lastechotime)
        lastechotime = dFileStr(fnum).utc;
    end
    echoTimeutc_full = lastechotime + (0:numframes-1)*tau;
    echoTimeutc_coh = lastechotime + tau*numCohAvg/2 + (0:numframes/numCohAvg-1)*tau*numCohAvg;
    echoTimeutc =   lastechotime + tau*numAvg/2 + (0:numframes/numAvg-1)*tau*numAvg;
    
    fid1 = fopen([inputfolder,dFileStr(fnum).name],'r');
    cleaner = onCleanup(@()fclose(fid1));
    tempFileData(excessframes*framelen+1:numframes*framelen) = ...
        fread(fid1,(framesToRead*framelen),'uint16','b');
    
    % Check data header as well as individual mean to make sure data frame
    % are properly aligned
    if checkHeader
        tempHeaders = tempFileData(headerind(1:numframes*framelen));
        if any(tempHeaders ~= 65535)
            warning('Data frames are not aligned')
            fixThisFile = 1;
            clear('cleaner');
            continue;
        end
    end
    tempData(:,1:numframes) = double(reshape(tempFileData(dataind(1:numframes*framelen)),fftdatalen,numframes));
    if checkDataAlign
        if any(abs(mean(tempData(:,1:numframes))-32767)>500)
            warning('Mean jumps in the data')
            fixThisFile = 1;
            clear('cleaner');
            continue;
        end
    end
    lastfileutc = dFileStr(fnum).utc;
    excessframes = numframesInFile - framesToRead;
    % update the laste echotime to time of firs excess echo
    lastechotime = echoTimeutc_full(end) + tau;
excessData(1:excessframes*framelen) = ...
        (fread(fid1,excessframes*framelen,'uint16','b'));
        clear('cleaner');
        
    lat = interp1(GPSUTC,Lat,echoTimeutc);
    lon = interpDegWithJumps(GPSUTC,Lon,echoTimeutc);
    hell = interp1(GPSUTC,Alt,echoTimeutc);
    pitch = interpDegWithJumps(GPSUTC,Pitch,echoTimeutc);
    roll = interpDegWithJumps(GPSUTC,Roll,echoTimeutc);
    heading = interpDegWithJumps(GPSUTC,Heading,echoTimeutc);
    if(isfirstfile == 1)
        isfirstfile = 0;
        firsttime = echoTimeutc(1);
        if (motionComp == 1)
            init_Alt = interp1(GPSUTC,Alt,echoTimeutc_coh(1));
        end
    end
    
    %% hilbert transform
    sig(:,1:numframes) = hilbert(detrend(tempData(:,1:numframes),'constant')*conversion_factor);
            %% Coherent Noise Removal i.e. DC doppler signal removal.
    if remove_meansig
%         if(lowMem ==1)
%             for k = 1:numframes
%                 sig(:,k) = sig(:,k)-meanSirSig;
%             end
%         else
%             sig(:,1:numframes) = sig(:,1:numframes)-meanSirMat(:,1:numframes);
%         end
%         [b,a] = iirnotch(0.000001,.00002);
%         sig(:,1:numframes) = ifft(filter(b,a,fft(sig(:,1:numframes),[],2),[],2),[],2);
        sig(:,1:numframes) = fft(sig(:,1:numframes),[],2);
        sig(:,[1,numframes]) = 0.0 + 0.0i;
        sig(:,1:numframes) = ifft(sig(:,1:numframes),[],2);
    end
    %% Coherent Averaging
    if(numCohAvg>1)
        sig(:,1:numframes/numCohAvg) = reshape(mean(reshape(sig(:,1:numframes),fftdatalen,numCohAvg,numframes/numCohAvg),2),fftdatalen,numframes/numCohAvg);
    end
       %% Filter 
    if usefilter
        sig(:,1:numframes/numCohAvg) = filtfilt(sirFilt,1,sig(:,1:numframes/numCohAvg));
    end
    %% Motion compensation
    % calculate H for motion compensation
    if(motionComp == 1)
        if ~lowMem
            del_tau(:,1:numframes/numCohAvg) = 2*(repmat((interp1(GPSUTC,Alt,echoTimeutc_coh) - init_Alt),fftdatalen,1))/c;
            H_MC(:,1:numframes/numCohAvg) = exp(-1j*2*pi*(f0*del_tau(:,1:numframes/numCohAvg)+ ...
                Kr.*tsampMat_full(:,1:numframes/numCohAvg).*del_tau(:,1:numframes/numCohAvg) - ...
                1/2*Kr*del_tau(:,1:numframes/numCohAvg).^2));
            sig(:,1:numframes/numCohAvg) = sig(:,1:numframes/numCohAvg).* H_MC(:,1:numframes/numCohAvg);
        else
            del_tau(1:numframes/numCohAvg) = 2*(interp1(GPSUTC,Alt,echoTimeutc_coh) - init_Alt)/c;
            for ll = 1:fftdatalen
                sig(ll,1:numframes/numCohAvg) = sig(ll,1:numframes/numCohAvg).*exp(-1j*2*pi*(f0*del_tau(1:numframes/numCohAvg)+ ...
                    Kr.*tsamp(ll).*del_tau(1:numframes/numCohAvg) - ...
                    1/2*Kr*del_tau(1:numframes/numCohAvg).^2));
            end
        end
    end
    %% Power Calc
    sirPwrEcho(1:numframes/numCohAvg) = rms(sig(:,1:numframes/numCohAvg));
    if normalizeEchoes
        if ~lowMem
            sig(:,1:numframes/numCohAvg) = sig(:,1:numframes/numCohAvg)./repmat(sirPwrEcho(1:numframes/numCohAvg),fftdatalen,1);
        else
%             for k = 1:fftdatalen
%                 sig(k,1:numframes/numCohAvg) = sig(k,1:numframes/numCohAvg)./sirPwrEcho(1:numframes/numCohAvg);
%             end
            
            sig(k,1:numframes/numCohAvg) = bsxfun(@rdivide,sig(:,1:numframes/numCohAvg),sirPwrEcho(1:numframes/numCohAvg));
        end
    end
    %% Apply residual video phase correction
    if(rvp_corr == 1)
        if(lowMem ==1)
            sig(:,1:numframes/numCohAvg) = fftshift(fft(sig(:,1:numframes/numCohAvg)),1);
%             for k=1:numframes/numCohAvg
%                 sig(:,k) = sig(:,k).*rvpCorr;
%             end
                sig(:,1:numframes/numCohAvg) = bsxfun(@times,sig(:,1:numframes/numCohAvg),rvpCorr);
            sig(:,1:numframes/numCohAvg) = ifft(ifftshift(sig(:,1:numframes/numCohAvg),1));
        else
            sig(:,1:numframes/numCohAvg) = ifft(ifftshift(fftshift(fft(sig(:,1:numframes/numCohAvg)),1).*rvpCorrMat(:,1:numframes/numCohAvg),1));
        end
    end
    %% FFT
    if(lowMem ==1)
%         for k = 1:numframes/numCohAvg
%             sif2(:,k) = fft(fftwin.*sig(:,k),NFFTr);
%         end
            sif2(:,1:numframes/numCohAvg) = fft(bsxfun(@times,sig(:,1:numframes/numCohAvg),fftwin),NFFTr);
    else
        if(gpu_process == 1)
            tempFFTdata(:,1:numframes/numCohAvg) = fftwinMat(:,1:numframes/numCohAvg).*sig(:,1:numframes/numCohAvg);
            sif2(:,1:numframes/numCohAvg) = gather(fft(tempFFTdata(:,1:numframes/numCohAvg),NFFTr));
        else
            sif2(:,1:numframes/numCohAvg) = fft(fftwinMat(:,1:numframes/numCohAvg).*sig(:,1:numframes/numCohAvg),NFFTr);
        end
    end
    if(numfftAvg>1)
        sirfftout = reshape(mean(reshape(sif2(InRange,1:numframes/numCohAvg),numel(InRange),numfftAvg,numframes/numCohAvg/numfftAvg),2),length(InRange),numframes/numCohAvg/numfftAvg);
        sirPwrOut = mean(reshape(sirPwrEcho(1:numframes/numCohAvg),numfftAvg,numframes/numCohAvg/numfftAvg),1);
    else
        sirfftout = sif2(InRange,1:numframes/numCohAvg);
        sirPwrOut = sirPwrEcho(1:numframes/numCohAvg);
    end
    %%
    if findLayers
        layerWinInd = find(Range>= layerWin(1) & Range <= layerWin(2));
        [layerVal,layerInd] = findpeaksMat(intuitive_lowpass2(20*log10(abs(sirfftout(layerWinInd,1:numframes/numAvg))),layerSmoothWin,1),numLayers,'descend');
        layerInd = layerWinInd(1) + layerInd -1;
    else
        layerInd = NaN;
        layerVal = NaN;
    end
    %%
    if(SaveData == 1)
        save([outputfolder,Campaign,'_',FlightNo,'_',datestr(unix2matlabtime(echoTimeutc(1)),'yyyymmdd_HHMMSS')],...
            'Campaign', 'FlightNo','FlightDesc','sirfftout','fs','fbins','NFFTr','tsamp','Range','echoTimeutc','fftwintype','fftwinparam','firsttime','numfftAvg','numCohAvg','sirPwrOut','twowaytime',...
            'lat','lon','hell','pitch','roll','heading','layerInd','layerVal');
    end
    
    if plotFigure
        sirfftout = detrend(log10(abs(sirfftout)),'constant');
        if smooth_plot
            if any(any(isnan(sirfftout) | isinf(sirfftout)))
                sirfftout(isnan(sirfftout) | isinf(sirfftout)) = mean(mean(sirfftout(~isnan(sirfftout) & ~isinf(sirfftout))));
            end
            sirfftout =  intuitive_lowpass2(sirfftout,smoothwin_x,smoothwin_y);
        end
        if diffplot
            sirfftout = diff(sirfftout,difforder,1);
            echoTimeutc = echoTimeutc(1:end-1);
        else
            sirfftout = -20*sirfftout;
        end
        if ~ishghandle(figHandle);
            figHandle = figure(fignum);
        end
        imagesc(unix2matlabtime(echoTimeutc),Range,sirfftout)
        hold on
        if exist('cLims','var')
            caxis(cLims);
        end
        if findLayers && ~isempty(plotLayers)
            for kk = 1:numel(plotLayers)
                plot(unix2matlabtime(echoTimeutc),Range(layerInd(plotLayers(kk),:)),layerLineSpec{plotLayers(kk)})
            end
            legend(layerName(plotLayers))
        end
        xlim(unix2matlabtime([firsttime,echoTimeutc(end)]))
        ylim([min(Range),max(Range)])
        datetick2('x','HH:MM:SS','keeplimits')
        colormap gray
        title(titletext);
        ylabel('Range [m]');
        xlabel('UTC time of Day');
        if (exist('ylims','var'))
            ylim(ylims)
        end
        if contFigUpdate
            drawnow;
        end
    end
    fnum = fnum +1;
end
toc
%% Save figure:
if plotFigure ==1
    datetick2('x','HH:MM:SS','keeplimits')
    if SaveFigure == 1
        figname = [outputfolder,'SIR-',FlightNo,'-',FlightDesc,num2str(numCohAvg,'-CohAvg=%d')...
            num2str(numfftAvg,'-fftAvg=%d-'),starttime(1:10),'-',...
            datestr(datenum(starttime(end-8:end)),'HH.MM'),'-',...
            datestr(datenum(endtime(end-8:end)),'HH.MM'),'-',num2str(now),'.png'];
        pagesize=[15,8];                   % inches
        resolution=600;                    % dpi
        set(gcf,'PaperSize',pagesize)
        set(gcf,'PaperPosition',[0,0,pagesize])
        print(figHandle,'-dpng',figname,['-r',num2str(resolution)])
    end
end